/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * FXDisplay.h: test effect
 * leto/calodox, 12.2000
 */

#include "opengl.h"
#include <GL/glu.h>
#include <stdio.h>
#include <string.h>
#include "fxdisplay.h"

FXDisplay::FXDisplay() : buff(0) {
	filtering = 0;
	nberr=0;
}

FXGeneric* FXDisplay::create() {
	return new FXDisplay();
}

void FXDisplay::errChk(int res) {
	GLenum err = glGetError();
	if (err) {
		nberr++;
		fprintf(stderr, "%s:%d: OpenGL error: %s\n", __FILE__, res, gluErrorString(err));
	}
}

void FXDisplay::registerOptions() {
	registerDrawBufferOpt("Source buffer", 0, &buff);
	registerBoolOpt("Texture filtering", OPTION_REINIT, &filtering);
	registerRectangleOpt("Position", 0, rect);
}
	
void FXDisplay::setupTex() {
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	if (filtering) {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	} else {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	}
}

int FXDisplay::init() {
	// no z-buffering
	glDisable(GL_DEPTH_TEST);


	// we need no blending for texturing.
	//glEnable(GL_BLEND);
	glDisable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// pixel storage mode
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	//glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    
	glGenTextures(1,&textName);

	glBindTexture(GL_TEXTURE_2D, textName); // prepare the fb texture

	setupTex();

	int texBuffer[512*512];
	memset(texBuffer, 0, sizeof(texBuffer));
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512,512, 0, GL_RGBA,
		GL_UNSIGNED_BYTE, texBuffer);
	
	return 0;


}

int FXDisplay::render() {

	float x = buff->x >= 512 ? 512.0 : (float) buff->x;
	float y = buff->y >= 512 ? 512.0 : (float) buff->y;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// gouraud, no light computation from GL
	glShadeModel(GL_SMOOTH);
	glDisable(GL_LIGHTING);
	
	// get ready to blit
	glBindTexture(GL_TEXTURE_2D, textName);

	// finished triangles. draw the pixels.

	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (int)x, (int)y,
			GL_RGBA, GL_UNSIGNED_BYTE, buff->buff);

	glEnable(GL_TEXTURE_2D);

	glEnable(GL_BLEND);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	glBegin(GL_QUADS);
	glColor4f(1,1,1,1);

	glTexCoord2f(0,0);
	glVertex2f(rect[0]*2-1, 1-rect[2]*2);

	glTexCoord2f(0,y/512.0);
	glVertex2f(rect[0]*2-1, 1-rect[3]*2);

	glTexCoord2f(x/512.0,y/512.0);
	glVertex2f(rect[1]*2-1, 1-rect[3]*2);

	glTexCoord2f(x/512.0,0);
	glVertex2f(rect[1]*2-1, 1-rect[2]*2);

	glEnd();

	glDisable(GL_TEXTURE_2D);
	return 0;
}

void FXDisplay::finish() {
}
