/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * FXFire.h: simple fire effect for demoniac
 * leto@calodox.org, 9.2001
 */

#include <stdlib.h> // for rand()
#include <string.h> // for memcmp()
#include <fx.h>
#include "fxfire.h"
#include <stdio.h>

FXFire::FXFire() : buff(0), fireBuff(0), falloff(1024) {
	col[0] = lastCol[0] = 0;
	col[1] = lastCol[1] = 0;
	col[2] = lastCol[2] = 1;
	col[3] = lastCol[3] = 1;
	col2[0] = lastCol2[0] = 1;
	col2[1] = lastCol2[1] = 0;
	col2[2] = lastCol2[2] = 0;
	col2[3] = lastCol2[3] = 1;
}

FXGeneric *FXFire::create() {
	return new FXFire();
}

char *FXFire::getName() {
		return "Fire";
}

void FXFire::registerOptions() {
	registerColorOpt("First color", 0, col);
	registerColorOpt("Second color", 0, col2);
	registerDrawBufferOpt("Destination Buffer", 0, &buff);
	registerIntOpt("Fall off", 0, &falloff);
}

int FXFire::init() {
		return 0; 
}

int FXFire::render() {
	int x;
	
	if (!buff) 
		return 0;

	// allocate a temporary buffer
	if (fireBuff == 0 || fireBuffX != buff->x || fireBuffY != buff->y) {
		if (fireBuff)
			delete[] fireBuff;
		fireBuffX = buff->x;
		fireBuffY = buff->y;
		fireBuff = new unsigned int[fireBuffX * fireBuffY];
	}


	// randomize the first line
	for (x = 0; x < fireBuffX; x++) 
		fireBuff[(fireBuffY-1)*fireBuffX + x] = ((unsigned int) 
				(((float)rand()* (float)0x1FFFF) / (float)RAND_MAX));

	// compute the fire itself
	for (int y= buff->y-2; y!=0; y--) {
		for (x=0; x<fireBuffX; x++) {
			unsigned int tmp = fireBuff[(y+1)*fireBuffX + x -1];
			tmp += 2*fireBuff[(y+1)*fireBuffX + x];
			tmp += fireBuff[(y+1)*fireBuffX + x + 1];
			tmp += fireBuff[y*fireBuffX + x - 1];
			tmp += 2*fireBuff[y*fireBuffX + x];
			tmp += fireBuff[y*fireBuffX + x + 1];
			fireBuff[y*fireBuffX + x] = (tmp > falloff ? tmp - falloff :0)/8 ;
		}
	}

	static bool first = true;

	// build the pallette, if necessary
	if (first ||memcmp(col, lastCol, sizeof(col)) ||
		    memcmp(col2, lastCol2, sizeof(col))) {
		first = false;
		for (int i=128; i<256; i++) {
			float f = (((float) (i-128)) / 128.0f) * 255.0f;
			pal[i].rgb.r = (unsigned char) (f  * col2[0] + (255.0f - f)*col[0]);
			pal[i].rgb.g = (unsigned char) (f  * col2[1] + (255.0f - f)*col[1]);
			pal[i].rgb.b = (unsigned char) (f  * col2[2] + (255.0f - f)*col[2]);
			pal[i].rgb.a = 255;
		}
		for (int j=0; j<128; j++) {
			float f = (((float) j)/127.0f) * 255.0f;
			pal[j].rgb.r = (unsigned char) (f  * col[0]);
			pal[j].rgb.g = (unsigned char) (f  * col[1]);
			pal[j].rgb.b = (unsigned char) (f  * col[2]);
			pal[j].rgb.a = 255;
		}
	}

	// fill the destination buffer with the pallette
	for (int i=0; i< buff->x * buff->y; i++) {
		unsigned int index = fireBuff[i] >> 8;
		buff->buff[i] = pal[index > 255 ? 255 : index];
	}

	return 0;
}

void FXFire::finish() {
}

