/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Parametric surface effect
 * Inspired by MoralHardCandy 
 *
 * Leto/calodox, 6 2001
 */

// for debugging
#include <stdio.h>

#include <math.h>
#include "opengl.h"
#include <string.h>
#include <math.h>
#include <fx.h>
#include "fxparam.h"

// half size of particles
#define PARTIC_SIZE .1
#define PARTIC_CENTER 2

#define TEXTURE_SIZE 64

#ifndef M_PI
#define M_PI 3.1415926535897932384626433832795
#endif 

/*
 * FXGeneric interface
 */

FXParam::FXParam() :
	nbU(10), nbV(10), zoom(1), surface(0), surface2(0),
	shellN(5), shellA(0.6), shellB(6), shellC(0.6),
	hypoA(2), hypoB(1)
{
	pos[0]=0;
	pos[1]=0;
	pos[2]=-2;

	color[0] = color[1] = color[2] = color[3] =1;

	for (unsigned int i=0; i<4; i++)
		for(unsigned int j=0; j<4; j++)
			if (i==j) rot[i][j]=1; 
			else rot[i][j]=0;

}

FXGeneric *FXParam::create() {
	return new FXParam();
}

void FXParam::registerOptions() {
	registerColorOpt("Color", 0, color);
	registerIntOpt("U segments", 0, &nbU, 1);
	registerIntOpt("V segments", 0, &nbV, 1);
	register3DPosOpt("Position",0,pos);
	register3DRotOpt("Rotation",0,rot);
	registerIntOpt("surface 1", 0, &surface, 0, 4);
	registerIntOpt("surface 2", 0, &surface2, 0, 4);
	registerFloatOpt("Morphing", 0, &morph, 0,1);
	registerFloatOpt("Sphere zoom", 0, &zoom, FLT_MIN);
	registerIntOpt("Shell: number of spirals", 0, &shellN,1);
	registerFloatOpt("Final shell radius", 0, &shellA);
	registerFloatOpt("Shell inner radius", 0, &shellC);
	registerFloatOpt("height", 0, &shellB);
	registerFloatOpt("Hypocycloid: outer radius", 0, &hypoA);
	registerFloatOpt("Hypocycloid: inner radius", 0, &hypoB);

	
	for (unsigned char i=0; i< 8; i++) {
		char str[] = "Sph. Harm. x";
		str[11] = '0' + i;
		registerIntOpt(str, 0, m + i, 0);
	}
}

// initialisation
int FXParam::init() {
	genPartImage();
	return 1;
}

int FXParam::render() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-1,1,-1,1,1,20);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(pos[0], pos[1], pos[2]);
	glMultMatrixf(&rot[0][0]);

	particles();
	singleLine();
	return 1;
}

/*
 * Methods of filling
 */
void FXParam::singleLine() {
	Vect2D uv = {0, 0} ;
	Vect3D last; 

	getSurface(uv, last);
	

	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glBegin(GL_LINE_STRIP);
	for (int iv=0; iv<nbV; iv++) {
		uv[0] = 0;
		for (int iu=0; iu<nbU; iu++) {
			Vect3D next;

			getSurface(uv,next);
			
			// draw the line
			glColor4f(color[0], color[1], color[2], color[3]);
			glVertex3f(next[0], next[1], next[2]);
			
			// get ready for the next one
			//memcpy(last, next, sizeof(Vect3D));
			uv[0]+= 1.0f/(float)nbU;
			uv[1]+= 1.0f/((float)nbV*(float)nbU);
		}
	}
	glEnd();

}

void FXParam::particles() {
	float mat[4][4];

	glDepthMask( GL_FALSE );
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);
	glMatrixMode(GL_MODELVIEW);
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureName);

	glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat *)mat);
	Vect3D v, h;
	for (int i=0; i<3;i++) {
		h[i] = mat[i][0] * PARTIC_SIZE;
		v[i] = mat[i][1] * PARTIC_SIZE;
	}

	Vect2D uv = {0, 0} ;
	
	glBegin(GL_QUADS);
	for (int iv=0; iv<nbV; iv++) {
		uv[0] = 0;
		for (int iu=0; iu<nbU; iu++) {

			Vect3D pos;

			getSurface(uv, pos);

			glColor4f(color[0], color[1], color[2], color[3]);
			glTexCoord2f(0,0);
			glVertex3f(pos[0]-h[0] -v[0], pos[1]-h[1] -v[1], pos[2]-h[2] -v[2]);

			glTexCoord2f(0,0.99);
			glVertex3f(pos[0]-h[0] +v[0], pos[1]-h[1] +v[1], pos[2]-h[2] +v[2]);

			glTexCoord2f(.99,.99);
			glVertex3f(pos[0]+h[0] +v[0], pos[1]+h[1] +v[1], pos[2]+h[2] +v[2]);

			glTexCoord2f(.99,0);
			glVertex3f( pos[0]+h[0] -v[0], pos[1]+h[1] -v[1], pos[2]+h[2] -v[2]);
			uv[0]+= 1.0f/(float)nbU;
			uv[1]+= 1.0f/((float)nbV*(float)nbU);
		}
	}
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glDepthMask( GL_TRUE);
}

void FXParam::filled() {
	glBegin(GL_TRIANGLES);
	glColor4f(color[0], color[1], color[2], color[3]);

	for (int iv=0; iv<nbV; iv++) {
		for (int iu=0; iu<nbU; iu++) {
			Vect2D uv[4];
			uv[0][0] = ((float)iu)/((float)nbU);
			uv[0][1] = ((float)iv)/((float)nbV);

			uv[1][0] = ((float)iu+1)/((float)nbU);
			uv[1][1] = ((float)iv+1)/((float)nbV);

			uv[2][0] = uv[0][0];
			uv[2][1] = uv[1][1];
			
			uv[3][0] = uv[1][0];
			uv[3][1] = uv[0][1];

			Vect3D v;
			getSurface(uv[0], v);
			glVertex3f(v[0], v[1], v[2]);
			getSurface(uv[2], v);
			glVertex3f(v[2], v[1], v[2]);
			getSurface(uv[1], v);
			glVertex3f(v[1], v[1], v[2]);
		}
	}
	glEnd();
}




void FXParam::finish() {
}

void FXParam::getSurface(Vect2D f, Vect3D dest) {
	Vect3D v1, v2;
	switch(surface) {
		case 0: sphere(f, v1); break;
		case 1: shell(f,v1); break;
		case 2: hypocycloid(f, v1); break;
		case 3: kleinCycloid(f, v1); break;
		case 4: sphHarm(f, v1); break;
	}
	switch(surface2) {
		case 0: sphere(f, v2); break;
		case 1: shell(f,v2); break;
		case 2: hypocycloid(f, v2); break;
		case 3: kleinCycloid(f, v2); break;
		case 4: sphHarm(f, v2); break;
	}

	for( int i=0; i<3; i++) 
		dest[i] = v1[i] * morph + (1-morph) * v2[i];
}


/*
 * parametric sphere
 */

void FXParam::sphere(Vect2D f, Vect3D dest) {
	float u = f[0]*2.0*M_PI;
	float v = f[1]*M_PI;

	float cu = cos(u);
	float su = sin(u);
	float sv = sin(v);

	dest[0] = cu * sv *zoom;
	dest[1] = su * sv *zoom;
	dest[2] = cos(v) *zoom;

}

/*
 * parametric shell, from Paul Bourke.
 * (http://astronomy.swin.edu.au/pbourke/modelling/shell/index.html)
 */
void FXParam::shell(Vect2D f, Vect3D dest) {
	float s = f[0]*2.0*M_PI;
	float t = f[1]*2.0*M_PI;

	dest[0] = shellA*(1-f[1])*cos(shellN*t)*(1+cos(s))+shellC*cos(shellN*t);
	dest[1] = shellA*(1-f[1])*sin(shellN*t)*(1+cos(s))+shellC*sin(shellN*t);
	dest[2] = shellB*f[1] + shellA *(1-f[1])*sin(s);
}

// http://www-history.mcs.st-and.ac.uk/history/Curves/Hypocycloid.html
void FXParam::hypocycloid(Vect2D f, Vect3D dest) {

	float t = f[0] *2*M_PI  - M_PI;

	dest[0] = (hypoA - hypoB) * cos(t) + hypoB * cos( (hypoA/hypoB - 1) * t );
	dest[1] = (hypoA - hypoB) * sin(t) - hypoB * sin( (hypoA/hypoB - 1) * t);

	dest[0] *= f[1];
	dest[1] *= f[1];
	dest[2] = f[1] * shellB;
}

// Klein Cycloid
// http://astronomy.swin.edu.au/pbourke/geometry/kleincycloid/
void FXParam::kleinCycloid(Vect2D f, Vect3D dest) {
	float a = shellA;
	float b = shellB;
	float c = shellC;
	float u = f[0] * 2* a * c * M_PI;
	float v = f[1] * 4 * M_PI;

	dest[0] = cos(u/c) * cos(u/b) * (a + cos(v)) + sin(u/b) * sin(v) * cos(v);
	dest[1] = sin(u/c) * cos(u/b) * (a + cos(v)) + sin(u/b) * sin(v) * cos(v);
	dest[2] = -sin(u/b) * (a + cos(v)) + cos(u/b) * sin(v) * cos(v);
}

void FXParam::sphHarm(Vect2D f, Vect3D dest) {
	float phi = f[0] * M_PI;
	float theta = f[1] * 2 *M_PI;

	float r =
		pow(sin(m[0]*phi),m[1]) + 
		pow(cos(m[2]*phi),m[3]) + 
		pow(sin(m[4]*theta),m[5]) + 
		pow(cos(m[6]*theta),m[7]);

	// change back from spherical coordinates
	dest[0] = r * cos(theta) * sin(phi);
	dest[1] = r * sin(theta) * sin(phi);
	dest[2] = r * cos(phi);
	
}

/*
 * Particle bitmap generation
 */
void FXParam::genPartImage() {

	glGenTextures(1, &textureName);
	glBindTexture(GL_TEXTURE_2D, textureName);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	// texture filtering is enabled 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	GLfloat texture[TEXTURE_SIZE][TEXTURE_SIZE][4];


	for (int y=0; y<TEXTURE_SIZE; y++) {
		for (int x=0; x<TEXTURE_SIZE; x++) {
			float sqrDist = sqrt( ((x-(TEXTURE_SIZE/2))*(x-(TEXTURE_SIZE/2)) +
				(y-(TEXTURE_SIZE/2))*(y-(TEXTURE_SIZE/2))));

			if (PARTIC_CENTER  >= sqrDist) {

				texture[y][x][3] =  // alpha channel
				texture[y][x][0] = 
				texture[y][x][1] =
				texture[y][x][2] = 1; // almost white
			} else {
				float fact = (float)(sqrDist - PARTIC_CENTER) / (float)((TEXTURE_SIZE/2)-1-PARTIC_CENTER);
				fact = 1-fact;

				texture[y][x][3] =  // alpha channel
				texture[y][x][0] = 
				texture[y][x][1] = 
				texture[y][x][2] = fact;
			}

			
			if (sqrDist >= ((TEXTURE_SIZE/2)-1)) {
				texture[y][x][3] =  // alpha channel
				texture[y][x][0] = 
				texture[y][x][1] = 
				texture[y][x][2] = 0;
			}
		}
	}

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, TEXTURE_SIZE, TEXTURE_SIZE, 
			0, GL_RGBA, GL_FLOAT, (void *)texture);

}
