/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Parametric surface effect
 * Inspired by MoralHardCandy 
 *
 * Leto/calodox, 6 2001
 */

#ifndef _FXPARAM_H_
#define _FXPARAM_H_

#include <fx.h>


class FXParam : public FXGeneric {
private:
	// number of segments for u and v
	// (nbU*nbV lines will be drawn)
	int nbU, nbV;
	float zoom;
	int surface, surface2;
	int shellN;
	float shellA, shellB, shellC;
	float hypoA, hypoB;
	int m[8]; // for spherical harmonics
	float pos[4];
	float rot[4][4];
	float morph;

	// for particle texturing
	unsigned int textureName;

	typedef float Vect3D[3];
	typedef float Vect2D[2];

	float color[4];

	/*
	 * Methods of drawing
	 */

	void singleLine();
	void filled();
	void genPartImage();
	void particles();

	/*
	 * all surface functions
	 */
	void getSurface(Vect2D f, Vect3D dest);
	void sphere(float *f, Vect3D dest);
	void shell(float *f, Vect3D dest);
	void hypocycloid(Vect2D f, Vect3D dest);
	void kleinCycloid(Vect2D f, Vect3D dest);
	void sphHarm(Vect2D f, Vect3D dest);

public:
		FXParam();

		virtual FXGeneric *create();

        virtual char *getName() { return "Parametric surface"; }

        virtual void registerOptions();

        virtual int init();

        virtual int render();

        virtual void finish();

};

#endif
