/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * FXTest.h: test effect
 * leto/calodox, 12.2000
 */

#include "opengl.h"
#include <stdio.h>
#include <string.h>
#include "fxtest.h"

FXTest::FXTest() {
	filtering = 0;
	nberr=0;
}

FXGeneric* FXTest::create() {
	return new FXTest();
}

void FXTest::errChk(int res) {
	if (res) {
		nberr++;
	}
}

void FXTest::registerOptions() {
	registerBoolOpt("Texture filtering", OPTION_REINIT, &filtering);
}
	
void FXTest::setupTex() {
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	if (filtering) {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	} else {
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	}
}

int FXTest::init() {
	// no z-buffering
	glDisable(GL_DEPTH_TEST);

	// gouraud, no light computation from GL
	glShadeModel(GL_SMOOTH);
	glDisable(GL_LIGHTING);

	// we need no blending for texturing.
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// pixel storage mode
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

    //glEnable(GL_TEXTURE_2D);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    
	glGenTextures(2,textName);
	glBindTexture(GL_TEXTURE_2D, textName[0]);
	setupTex();

	for(int y=0; y<256; y++) 
		for (int x=0; x<256; x++) {
			texture[y][x] = (x&64)^((y&64) ^ (x+y & 32)) ? 0xFFFFFF : 0x00;
			texture[y][x] |= 0xFF << 24;
			//texture[y][x] |= x << 16;
		}

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 256,256, 0, GL_RGBA,
		GL_UNSIGNED_BYTE, texture);

	glBindTexture(GL_TEXTURE_2D, textName[1]); // prepare the fb texture
	memset(texBuffer,0,512*512);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 512,512, 0, GL_RGBA,
		GL_UNSIGNED_BYTE, texBuffer);
	setupTex();
	
	return 0;


}

int FXTest::render() {
	float spec[3][4] = {
		{ 0,  0, .5,  0 },
		{ 0,  0, .1,  0.5 },
		{ 0,  0,  1,  0.6 }
	};
	int offset;

	glBindTexture(GL_TEXTURE_2D, textName[0]);

    glEnable(GL_TEXTURE_2D);
	
	// les triangles ne sont pas transparents.
	glDisable(GL_BLEND);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glBegin(GL_TRIANGLES);

	glColor4f(1.0f, 0, 0, 0);
	glTexCoord2f(0,0);
	glVertex2i(10,10);
	
	glColor4f(0.7f, 0, 0,1);
	glTexCoord2f(1.9,0.3);
	glVertex2i(300,100);

	glColor4f(.9f, 0, 0,1);
	glTexCoord2f(0.5,0.9);
	glVertex2i(100,180);

	// during triangles creation, draw pixels
	// in the frame buffer:
	
	offset=0;
	memset(texBuffer, 0xFFFFFFFF, 320*4);
	for (int y=0;y<200;y++) {
		texBuffer[offset] = 0xFFFFFFFF;
		texBuffer[offset+319] = 0xFFFFFFFF;
		texBuffer[offset+1+y] = 0xFFFFFF00 | y;
		texBuffer[offset+y] = 0xFFFFFF00 | y;
		offset+=320;
	}
	memset(texBuffer+199*320, 0xFFFFFFFF, 320*4);

	glEnd(); 

	// finished triangles. draw the pixels.
	glBindTexture(GL_TEXTURE_2D, textName[1]);
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 320, 200,
		GL_RGBA, GL_UNSIGNED_BYTE, texBuffer);

	glEnable(GL_BLEND);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	glBegin(GL_QUADS);
		glColor4f(1,1,1,1);

		glTexCoord2f(0,0);
		glVertex2i(0,0);

//		glTexCoord2f(0,1);
		glTexCoord2f(0,200.0/512.0);
		glVertex2i(0,200);

//		glTexCoord2f(1,1);
		glTexCoord2f(320.0/512.0,200.0/512.0);
		glVertex2i(320,200);

//		glTexCoord2f(1,0);
		glTexCoord2f(320.0/512.0,0);
		glVertex2i(320,0);

	glEnd();

	glDisable(GL_TEXTURE_2D);
	return 0;
}

void FXTest::finish() {
}
