/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Time & effects control dialog for demoniac
 * leto@calodox.org, 9.2001
 */

#include <gtk/gtk.h>
#include "glmain.h"
#include "ctrldlg.h"
#include "report.h"

ControlDlg::ControlDlg()  
{
}

ControlDlg::~ControlDlg() {
}

void ControlDlg::build() {
	window = gtk_window_new( GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "DemoNiac controls");

	gtk_signal_connect(GTK_OBJECT(window), "delete_event", 
		GTK_SIGNAL_FUNC(Dlg::cbDeleteWindow), (Dlg *) this);

	// When a key is pressed.
	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
			GTK_SIGNAL_FUNC(cbKeyboard), NULL);
	
	// build the contents of the dialog
	GtkWidget *vboxCont = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vboxCont);

	// hbox for demo time label, spinner & play/stop button
	GtkWidget *hboxTime = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vboxCont), hboxTime, FALSE, TRUE, 5);

	GtkWidget *label = gtk_label_new("Demo time");
	gtk_box_pack_start(GTK_BOX(hboxTime), label, FALSE, FALSE, 5);
	gtk_widget_show(label);

	GtkAdjustment *timeAdjustment = glMainptr->getTimeAdjustment();

	timeSpinner = gtk_spin_button_new(timeAdjustment, 1, 3);
	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(timeSpinner), TRUE);
	gtk_signal_connect(GTK_OBJECT(timeAdjustment), "value_changed",
		GTK_SIGNAL_FUNC(cbDemoTime), timeSpinner);
	gtk_box_pack_start(GTK_BOX(hboxTime), timeSpinner, TRUE, TRUE, 5);
	gtk_widget_show(timeSpinner);

	GtkWidget *playStopButton = gtk_button_new();
	playStopButtonLabel = gtk_label_new("Play");
	gtk_widget_show(playStopButtonLabel);
	gtk_container_add(GTK_CONTAINER(playStopButton), playStopButtonLabel);
	gtk_box_pack_start(GTK_BOX(hboxTime), playStopButton, TRUE, TRUE, 5);
	gtk_signal_connect(GTK_OBJECT(playStopButton), "clicked",
			GTK_SIGNAL_FUNC(cbPlayStopButton), this);
	gtk_widget_show(playStopButton);

	// Create the effect selector
	GtkWidget *hboxFXChoose = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vboxCont), hboxFXChoose, FALSE, FALSE, 5);

	// create the effect menu
	GtkWidget *effectMenu = gtk_menu_new();
	for (unsigned int j=0; j<glMainptr->vfx.size(); j++) {
		
		GtkWidget *item = gtk_menu_item_new_with_label(glMainptr->vfx[j]->getName());
		gtk_menu_append(GTK_MENU(effectMenu), item);

		gtk_signal_connect(GTK_OBJECT(item), "activate",
				GTK_SIGNAL_FUNC(cbFXChoose), glMainptr->vfx[j]);
		gtk_widget_show(item);

	}
	gtk_widget_show(effectMenu);
	
	// create the option menu unrolling the effect menu
	GtkWidget *optMenu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(optMenu), effectMenu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(optMenu), 0);
	gtk_box_pack_start(GTK_BOX(hboxFXChoose), optMenu, TRUE, TRUE, 5);
	gtk_widget_show(optMenu);

	GtkWidget *newFXButton = gtk_button_new_with_label("New instance");
	gtk_signal_connect(GTK_OBJECT(newFXButton), "clicked",
			GTK_SIGNAL_FUNC(cbNewInstance), NULL);
	gtk_box_pack_start(GTK_BOX(hboxFXChoose), newFXButton, TRUE, TRUE, 5);
	gtk_widget_show(newFXButton);

	// column list of instances (in a scrolled window)
	GtkWidget *scrWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrWin),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vboxCont), scrWin, TRUE, TRUE, 0);

	gchar *titles[] = { "Act", "Instance name" , 0 };
	instCList = gtk_clist_new_with_titles(2, titles);
	gtk_clist_set_selection_mode(GTK_CLIST(instCList), GTK_SELECTION_BROWSE);
	gtk_clist_set_shadow_type (GTK_CLIST(instCList), GTK_SHADOW_OUT);
	gtk_signal_connect(GTK_OBJECT(instCList), "select_row", 
			GTK_SIGNAL_FUNC(cbFXInstanceCList), (gpointer) this);
	gtk_signal_connect(GTK_OBJECT(instCList), "row_move", 
			GTK_SIGNAL_FUNC(cbCListRowMove), (gpointer) this);
	gtk_clist_set_column_width (GTK_CLIST(instCList), 0, 30);
	gtk_clist_set_reorderable(GTK_CLIST(instCList), TRUE);
	gtk_clist_set_use_drag_icons(GTK_CLIST(instCList), TRUE);

	// add the clist in the scrolled window
	gtk_container_add(GTK_CONTAINER(scrWin), instCList);
	gtk_widget_show(scrWin); 
	gtk_widget_show(instCList); 

	// list operations button
	GtkWidget *listBBox = gtk_hbox_new(TRUE, 5);
	gtk_box_pack_start(GTK_BOX(vboxCont), listBBox, FALSE, FALSE, 5);
	GtkWidget *rem = gtk_button_new_with_label("Remove");
	gtk_box_pack_start(GTK_BOX(listBBox), rem, TRUE, TRUE, 5);
	gtk_signal_connect(GTK_OBJECT(rem), "clicked",
			GTK_SIGNAL_FUNC(cbRemoveInstance), (gpointer) this);

	GtkWidget *conf = gtk_button_new_with_label("Configure");
	gtk_box_pack_start(GTK_BOX(listBBox), conf, TRUE, TRUE, 5);
	gtk_signal_connect(GTK_OBJECT(conf), "clicked",
			GTK_SIGNAL_FUNC(cbConfigureButton), (gpointer) this);

	GtkWidget *up = gtk_button_new_with_label("up");
	gtk_box_pack_start(GTK_BOX(listBBox), up, TRUE, TRUE, 5);
	gtk_signal_connect(GTK_OBJECT(up), "clicked",
			GTK_SIGNAL_FUNC(cbUpButton), (gpointer) this);

	GtkWidget *down = gtk_button_new_with_label("down");
	gtk_box_pack_start(GTK_BOX(listBBox), down, TRUE, TRUE, 5);
	gtk_signal_connect(GTK_OBJECT(down), "clicked",
			GTK_SIGNAL_FUNC(cbDownButton), (gpointer) this);
	
	gtk_widget_show_all(vboxCont);
}

const char *ControlDlg::getName() {
	return "Time & effects controls";
}

void ControlDlg::addFXInstance(FXInstance *fxi) {
	gchar *names[] = { "+", 0, 0 };
	names[1] = fxi->name;
	gtk_clist_append(GTK_CLIST(instCList), names);
}

void ControlDlg::renameInstance(unsigned int i) {
	gtk_clist_set_text(GTK_CLIST(instCList),
			i, 1, glMainptr->vfxi[i]->name);
}

void ControlDlg::removeListItem(unsigned int nb) {
	gtk_clist_remove(GTK_CLIST(instCList), nb);
}


void ControlDlg::setPlayStopLabel(const char *name) {
	gtk_label_set_text(GTK_LABEL(playStopButtonLabel), name);
}

/******************************************
 *           Callback functions           *
 *****************************************/

// a change in the effect listbox
gint cbFXChoose(GtkWidget *widget, gpointer data) {

	if (data == 0) return TRUE;
	glMainptr->selectedFX = (FXGeneric *) data;
	return TRUE;
}

// instance list box call back
gint cbFXInstanceCList(GtkWidget *widget, gint row, gint col,  GdkEvent *event, gpointer data) {
	GLMain *obj = glMainptr;

	if ((unsigned int)row < obj->vfxi.size())
		obj->selectedFXInstance = obj->vfxi[row];
	return TRUE;
}

// configure instance callback
gint cbConfigureButton(GtkWidget *widget, gpointer data) {
	GLMain *obj = (GLMain *) glMainptr;

	if (obj->selectedFXInstance == 0) return TRUE;

	obj->selectedFXInstance->dlg->show();
	return TRUE;
}

void cbCListRowMove(GtkCList *clist, gint a, gint b, gpointer data) {

	FXInstance *tmp = glMainptr->vfxi[a];
	glMainptr->vfxi.erase(glMainptr->vfxi.begin()+ a);
	glMainptr->vfxi.insert(glMainptr->vfxi.begin() +  b, tmp );

	glMainptr->reDraw();
#ifdef DEBUG
	printf("exchanged %d with %d. List is now:\n", a, b);
	for (unsigned int i=0; i<glMainptr->vfxi.size(); i++) 
		printf("%s\n", glMainptr->vfxi[i]->name);
#endif
}

gint cbUpButton(GtkWidget *widget, gpointer data) {
	ControlDlg *obj = (ControlDlg *) data;

	if (glMainptr->selectedFXInstance == 0)
		return TRUE;

	for (unsigned int i=1; i<glMainptr->vfxi.size(); i++) {
		if (glMainptr->selectedFXInstance==glMainptr->vfxi[i]) {
			gtk_clist_row_move(GTK_CLIST(obj->instCList), i, i-1);
			return TRUE;
		}
	}
	return TRUE;
}

gint cbDownButton(GtkWidget *widget, gpointer data) {
	ControlDlg *obj = (ControlDlg *) data;

	if (glMainptr->selectedFXInstance == 0)
		return TRUE;

	for (unsigned int i=0; i<glMainptr->vfxi.size()-1; i++) {
		if (glMainptr->selectedFXInstance==glMainptr->vfxi[i]) {
			gtk_clist_row_move(GTK_CLIST(obj->instCList), i, i+1);
			return TRUE;
		}
	}
	return TRUE;
}

gint cbRemoveInstance(GtkWidget *widget, gpointer data) {
	if (glMainptr->selectedFXInstance)
		glMainptr->removeFXInstance(glMainptr->selectedFXInstance);
	// make sure the screen is correctly drawn
	glMainptr->reDraw();
	return TRUE;
}


