/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * DrawBuffer list for demoniac
 * leto@calodox.org, 9.2001
 */


#include <gtk/gtk.h>
#include <fx.h>
#include <assert.h>
#include "genericoption.h"
#include "optdrawbuff.h"
#include "drawbufflist.h"
#include "fxinstance.h"
#include "report.h"

DrawBufferList::DrawBufferList() : buffList(0), watch(0) 
{
}

DrawBufferList::~DrawBufferList() {
	clear();
}

void DrawBufferList::clear() {
	while (buffList.size()) {
		removeDrawBuffer(buffList[0]);
	}
}

DrawBuffer *DrawBufferList::addDrawBuffer(const char *name, int x, int y) {

	if (name == 0)
		name = "default";

	DrawBuffer *buff = query(name);

	if (!buff) {
		buff = new DrawBuffer();
		char *n= new char[strlen(name)];
		strcpy(n, name);
		buff->name = n;
		buff->x = x == 0 ? 512 : x;
		buff->y = y == 0 ? 384 : y;

		buff->buff = new t_color[buff->x*buff->y];

		buffList.push_back(buff);
		addToCList(buff);
	} else {
		if (x && y && (x!= buff->x || y != buff->y)) {
			// resize an existing buffer
			delete[] buff->buff;
			buff->x = x;
			buff->y = y;
			buff->buff = new t_color[buff->x*buff->y];
			addToCList(buff, gtk_clist_find_row_from_data(GTK_CLIST(clist), buff));
		}	
	}

	return buff;
}

void DrawBufferList::removeDrawBuffer(DrawBuffer *buff) {

	removeFromCList(buff);

	std::vector<DrawBuffer *>::iterator ite;
	for (ite = buffList.begin(); ite != buffList.end(); ite++) {

		if (buff == *ite) {
			buffList.erase(ite);
			delete buff;
			break;
		}
	}
}

DrawBuffer *DrawBufferList::query(const char *name) {

	std::vector<DrawBuffer *>::iterator ite;

	for (ite = buffList.begin(); ite != buffList.end(); ite++) {

		if (strcmp((*ite)->name, name)==0)
			return *ite;
	}
	return 0;
}

void DrawBufferList::watchOptDrawBuffer(OptDrawBuffer *opt) {

	watch = opt;

	gtk_label_set_text(GTK_LABEL(fxLabel), opt->owner->name);
	gtk_label_set_text(GTK_LABEL(optLabel), opt->name);
	gtk_widget_show(watchVBox);
	show();
}

const char *DrawBufferList::getName() {
	return "All buffers";
}

void DrawBufferList::build() {
	char *titles[] = { "Name", "X size", "Y size", 0 };
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), getName());
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(Dlg::cbDeleteWindow), (Dlg *) this);

	GtkWidget *vbox = gtk_vbox_new(false, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	// scrolled window for the buffer list
	GtkWidget *scrWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrWin),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox), scrWin, TRUE, TRUE, 0);
	
	clist = gtk_clist_new_with_titles(3, titles);
	gtk_clist_set_sort_column(GTK_CLIST(clist), 0);
	gtk_clist_set_auto_sort(GTK_CLIST(clist), true);
	gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, true);
	gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 1, true);
	gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 2, true);
	rowSelected= -1;
	gtk_signal_connect(GTK_OBJECT(clist), "select_row",
			GTK_SIGNAL_FUNC(cbRowSelect), this);
	gtk_container_add(GTK_CONTAINER(scrWin), clist);


	std::vector<DrawBuffer *>::iterator ite;

	for (ite = buffList.begin(); ite != buffList.end(); ite++) {
		addToCList(*ite);
	}

	GtkWidget *frame = gtk_frame_new("Create / Modify buffer");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 5);
	
	GtkWidget *table = gtk_table_new(4,2, FALSE);
	gtk_container_add(GTK_CONTAINER(frame), table);

	GtkWidget *label = gtk_label_new("Name:");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
			(GtkAttachOptions) 0, (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 1, 1);
	nameEntry = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(table), nameEntry, 1, 2, 0, 1);

	label = gtk_label_new("X size:");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 
			(GtkAttachOptions) 0, (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 1, 1);
        // spin button
        GtkAdjustment *adj = (GtkAdjustment *) gtk_adjustment_new(
                (gfloat) 640, (gfloat)0, (gfloat)10000, 1, 10, 1);
        xSizeSpin = gtk_spin_button_new(adj, 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(xSizeSpin), TRUE);

	gtk_table_attach_defaults(GTK_TABLE(table), xSizeSpin, 1, 2, 1, 2);


	label = gtk_label_new("Y size:");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 
			(GtkAttachOptions) 0, (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 1, 1);
        // spin button
        adj = (GtkAdjustment *) gtk_adjustment_new(
                (gfloat) 480, (gfloat)0, (gfloat)10000, 1, 10, 1);
        ySizeSpin = gtk_spin_button_new(adj, 1, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(ySizeSpin), TRUE);

	gtk_table_attach_defaults(GTK_TABLE(table), ySizeSpin, 1, 2, 2, 3);

	GtkWidget *button = gtk_button_new_with_label("Create/Modify buffer");

	gtk_signal_connect(GTK_OBJECT(button), "clicked", 
			GTK_SIGNAL_FUNC(cbCreate), this);

	gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 2, 3, 4);


	// setup the option watch extension
	// stay hidden, waiting for an option to watch for.
	watchVBox = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), watchVBox, FALSE, FALSE, 0);
	fxLabel = gtk_label_new("<empty>");
	gtk_widget_show(fxLabel);
	gtk_box_pack_start(GTK_BOX(watchVBox), fxLabel, FALSE, FALSE, 0);
	optLabel = gtk_label_new("<empty>");
	gtk_widget_show(optLabel);
	gtk_box_pack_start(GTK_BOX(watchVBox), optLabel, FALSE, FALSE, 0);
	button = gtk_button_new_with_label("Set buffer");
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(cbSetBuffer), this);
	gtk_box_pack_start(GTK_BOX(watchVBox), button, FALSE, FALSE, 0);
	gtk_widget_show_all(button);


	gtk_widget_show_all(scrWin);
	gtk_widget_show_all(frame);
	gtk_widget_show(vbox);
}

/*
 * Private functions
 */

void DrawBufferList::addToCList(DrawBuffer *buff, int row) {

	assert(buff);

	char x[6], y[6];
	char *text[4];

	sprintf(x, "%5d", buff->x); x[5] = 0;
	sprintf(y, "%5d", buff->y); y[5] = 0;
	if (buff->name)
		text[0] = strdup(buff->name);
	else
		text[0] = "(unknown)";
	text[1] = x;
	text[2] = y;

	if (row < 0) {
		// insert a new row
		row = gtk_clist_append(GTK_CLIST(clist), text);
		gtk_clist_set_row_data(GTK_CLIST(clist), row, (gpointer) buff);
	} else {
		// just an update
		gtk_clist_set_text(GTK_CLIST(clist), row, 0, text[0]);
		gtk_clist_set_text(GTK_CLIST(clist), row, 1, text[1]);
		gtk_clist_set_text(GTK_CLIST(clist), row, 2, text[2]);
	}

	if (buff->name)
		free(text[0]);

}

void DrawBufferList::removeFromCList(DrawBuffer *buff) {
	int row = gtk_clist_find_row_from_data(GTK_CLIST(clist), (gpointer) buff);

	if (row != -1)
		gtk_clist_remove(GTK_CLIST(clist), row);
}
	
void DrawBufferList::cbCreate(GtkWidget *button, gpointer data) {

	DrawBufferList *obj = (DrawBufferList *) data;

	char *name = gtk_entry_get_text(GTK_ENTRY(obj->nameEntry));
	if (name && name[0] != 0) {
		int x = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(obj->xSizeSpin));
		int y = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(obj->ySizeSpin));

		obj->addDrawBuffer((const char *) name, x, y);
	}
}	

void DrawBufferList::cbSetBuffer(GtkWidget *widget, DrawBufferList *obj) {

	assert(obj->watch != 0);

	if (obj->rowSelected != -1) {
		DrawBuffer *buff = (DrawBuffer *) 
			gtk_clist_get_row_data(GTK_CLIST(obj->clist), obj->rowSelected);

		obj->watch->changeDrawBuffer(buff);
		gtk_widget_hide(obj->watchVBox);
		obj->watch = 0;
	}
}

void DrawBufferList::cbRowSelect(GtkWidget *widget, gint row, gint column, 
				GdkEventButton *event, gpointer data) {

	DrawBufferList *obj = (DrawBufferList *)data;
	
	obj->rowSelected = row;
	DrawBuffer *buff = (DrawBuffer *)
		gtk_clist_get_row_data(GTK_CLIST(obj->clist), row);

	// buff is 0 if the buffer has been freshly added and
	// the clist autoselect it.
	if (buff!=0) {

		gtk_entry_set_text(GTK_ENTRY(obj->nameEntry), buff->name);

		gtk_spin_button_set_value(GTK_SPIN_BUTTON(obj->xSizeSpin), buff->x);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(obj->ySizeSpin), buff->y);
	}
}

/*
 * Load / Save
 */

void DrawBufferList::save(Saver *s) {

	s->saveObjectBegin("DrawBufferList");

	std::vector<DrawBuffer *>::iterator ite;

	for (ite = buffList.begin(); ite != buffList.end(); ite++) {
		s->saveString("BufferName", (char *) (*ite)->name);
		s->saveInt("BufferXSize", (*ite)->x);
		s->saveInt("BufferYSize", (*ite)->y);
	}
	s->saveObjectEnd();
}

bool DrawBufferList::load(Loader *l) {


	if (!l->checkType(Loader::OBJECT) || !l->checkName("DrawBufferList")) 
		return false;

	l->nextLine();

	while ( l->nextType() == Loader::STRING ) {

		char name[128];
		int x, y;

		l->getString(name, sizeof(name), "BufferName");
		l->nextLine();
		x = l->getInt("BufferXSize");
		l->nextLine();
		y = l->getInt("BufferYSize");
		l->nextLine();

		addDrawBuffer(name, x, y);
	}

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
	return true;
}

