/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * DrawBuffer list for demoniac
 * leto@calodox.org, 9.2001
 */

#ifndef _DRAWBUFFLIST_H_
#define _DRAWBUFFLIST_H_

#include <vector>
#include <fx.h>
#include "dlg.h"
#include "saver.h"
#include "loader.h"

class OptDrawBuffer;

class DrawBufferList : public Dlg {
private:
	std::vector<DrawBuffer *> buffList;
	GtkWidget *clist;
	GtkWidget *nameEntry;
	GtkWidget *xSizeSpin, *ySizeSpin;

	OptDrawBuffer *watch;

	// GUI functions
	int rowSelected;

	void addToCList(DrawBuffer *buff, int row=-1);
	void removeFromCList(DrawBuffer *buff);

	static void cbCreate(GtkWidget *button, gpointer data);
	static void cbSetBuffer(GtkWidget *widget, DrawBufferList *obj);
	static void cbRowSelect(GtkWidget *widget, gint row, gint column, 
				GdkEventButton *event, gpointer data);

public:

	DrawBufferList();
	~DrawBufferList();

	// clear the list
	void clear();

	// returns a pointer to a buffer named 'name', which might
	// be freshly created.
	DrawBuffer *addDrawBuffer(const char *name, int x, int y);

	void removeDrawBuffer(DrawBuffer *buff);

	// 0 if not found.
	DrawBuffer *query(const char *name);

	void watchOptDrawBuffer(OptDrawBuffer *opt);

	// GUI functions
	virtual void build();
	virtual const char *getName();

	// GUI variables
	GtkWidget *watchVBox;
	GtkWidget *fxLabel, *optLabel;

	// load + save
	bool load(Loader *l);
	void save(Saver *s);
};
#endif

