/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Option: options for FXs in GLDemoniac.
 * Leto/calodox, 12.2000
 */

#include <string.h>
#include <assert.h>
#include <matvect.h>
#include "glmain.h"
#include "saver.h"
#include "genericoption.h"
#include "track.h"
#include "interpolator.h"
#include "optval.h"
#include "report.h"

gint cbInterpolator(GtkWidget *widget, gpointer data) {
	GenericOption *opt = (GenericOption *) data;

	if (!opt->track)
		return TRUE;

	for(int i=0; i< 4; i++) {
		if (opt->interpMenuItem[i] == widget) {
			opt->track->setInterpolator(getInterpByType(i+1));

			// take care of display/hide the option
			// in the track dialog
			if (i>0)
				glMainptr->trackDlg.addOption(opt);
			else
				glMainptr->trackDlg.removeOption(opt);

			return TRUE;
		}
	}
	debug("Warning: unknown interpolator.");
	return TRUE;
}

/*******************
 * Generic Option  *
 ******************/

// Constructor
GenericOption::GenericOption(FXInstance *owner, char *name, char *defName, int flags) :
	owner(owner),
	flags(flags),
	ignore(false) 
{
	setName(name, defName);
}

GtkBox *GenericOption::drawLabel(GtkBox *box) {
	ignore = false; 

	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

	// label
	GtkWidget *label = gtk_label_new(name);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
	
	return GTK_BOX(hbox);
}	


void GenericOption::drawInterp(GtkBox *box) {

	// create the menu 
	GtkWidget *menu = gtk_menu_new();
	gtk_widget_show(menu);

	interpMenuItem[0] = gtk_menu_item_new_with_label("Fixed value");
	gtk_widget_set_name(interpMenuItem[0], "1");
	
	gtk_signal_connect(GTK_OBJECT(interpMenuItem[0]), "activate", 
			GTK_SIGNAL_FUNC(cbInterpolator), this);
	gtk_widget_show(interpMenuItem[0]);
	gtk_menu_append(GTK_MENU(menu), interpMenuItem[0]);

	interpMenuItem[1] = gtk_menu_item_new_with_label("Last key");
	gtk_widget_set_name(interpMenuItem[1], "2");
	gtk_menu_append(GTK_MENU(menu), interpMenuItem[1]);
	gtk_signal_connect(GTK_OBJECT(interpMenuItem[1]), "activate", 
			GTK_SIGNAL_FUNC(cbInterpolator), this);
	gtk_widget_show(interpMenuItem[1]);

	if (track && track->canInterpolate()) {
		interpMenuItem[2] = gtk_menu_item_new_with_label("Linear");
		gtk_widget_set_name(interpMenuItem[2], "3");
		gtk_menu_append(GTK_MENU(menu), interpMenuItem[2] );
		gtk_signal_connect(GTK_OBJECT(interpMenuItem[2]), "activate", 
				GTK_SIGNAL_FUNC(cbInterpolator), this);
		gtk_widget_show(interpMenuItem[2]);

		interpMenuItem[3] = gtk_menu_item_new_with_label("Smooth");
		gtk_widget_set_name(interpMenuItem[3], "4");
		gtk_menu_append(GTK_MENU(menu), interpMenuItem[3] );
		gtk_signal_connect(GTK_OBJECT(interpMenuItem[3]), "activate", 
				GTK_SIGNAL_FUNC(cbInterpolator), this);
		gtk_widget_show(interpMenuItem[3]);
	}


	GtkWidget *optMenu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(optMenu), menu);
	gtk_box_pack_start(box, optMenu, FALSE, FALSE, 5);
	gtk_widget_show(optMenu);

	if (track)
		gtk_option_menu_set_history(GTK_OPTION_MENU(optMenu), 
			track->interpolator->getType()-1);
}

void GenericOption::setName (const char *newName, const char *def) {
	const char *str;

	str = newName ? newName : def;

	name = new char[strlen(str)+1];
	strcpy(name, str);
}

