/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * GLMain.cpp: GLDemoNiac main class 
 * Leto/calodox, 12.2000
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#else
#define PACKAGE_DATA_DIR "."
#define PACKAGE_SOURCE_DIR "../.."
#endif

#include "opengl.h"
#include <GL/glu.h>
#include <gtk/gtk.h>
#include <gtkgl/gtkglarea.h>
#include <gldemoniac.h>
#include "saver.h"
#include "loader.h"
#include "glmain.h"
#include "fxinstance.h"
#include "timer.h"
#include "menu.h"
#include "drawbufflist.h"
#include "pixmaps.h"

#include <assert.h>

// 10 seconds is the minimum demo duration
#define MINIMUM_DEMO_DURATION 10000

// global object pointer for callback functions and other classes.
GLMain *glMainptr=0;

/******************************************
**        GLDemoniac Hide Class          **
*****************************************/

GLDemoniac::GLDemoniac() {
	this->glMainPtr = new GLMain();
}

GLDemoniac::~GLDemoniac() {
	delete this->glMainPtr;
}

int GLDemoniac::registerFX(FXGeneric *fx, int fxsize) {
	return this->glMainPtr->registerFX(fx, fxsize);
}

int GLDemoniac::start(int *argc, char ***argv) {
	return this->glMainPtr->start(argc, argv);
}

void GLDemoniac::reDraw() {
	this->glMainPtr->reDraw();
}
	

/* Constructor: called before any fx registration */
GLMain::GLMain(void) {
	glMainptr = this;
	started = 0;
	selectedFX = 0;
	selectedFXInstance = 0;
	playing = false;
	demoTimeSec = 0.0;
	filename = 0;
}

// finish all effects
GLMain::~GLMain() {
	clearAll();
}

/***************************************
 *      DemoNiac Business Logic        *
 **************************************/

/*
 * Effect instances manipulation
 */

void GLMain::clearAll() {
	while (vfxi.size()) {
		removeFXInstance(vfxi[vfxi.size()-1]);
	}

	started = 0;

	selectedFXInstance = 0;
	if (playing) 
		cbPlayStopButton(0, this);
	if (filename) {
		delete[] filename;
		filename = 0;
	}
	setTime(0);

	drawBufferList.clear();
}

void GLMain::renameInstance(FXInstance *ptr) {
	if (ptr->treeItem) {
		GList *l = gtk_container_children(GTK_CONTAINER(ptr->treeItem));
		GtkWidget *w = (GtkWidget *) l->data;

		gtk_label_set_text(GTK_LABEL(w), ptr->name);
	}	

	for (unsigned int i=0; i< vfxi.size(); i++) {
		if (vfxi[i] == ptr) { 
			ctrlDlg.renameInstance(i);
			break;
		}
	}
}

void GLMain::addFXInstance(FXInstance *fxi) {
	vfxi.push_back(fxi);

	ctrlDlg.addFXInstance(fxi);
	trackDlg.addFXInstance(fxi);

	if (gtk_gl_area_make_current(GTK_GL_AREA(glarea))) 
		fxi->init();
}

bool GLMain::removeFXInstance(FXInstance *fxi) {
	for (unsigned int i=0; i< vfxi.size(); i++) {
		if (vfxi[i] == fxi) { 
			vfxi.erase(vfxi.begin() + i);

			if (selectedFXInstance == fxi) 
				selectedFXInstance = 0;

			ctrlDlg.removeListItem(i);
			trackDlg.removeFXInstance(fxi);

			delete fxi;
			return true;
		}
	}
	return false;
}

int GLMain::registerFX(FXGeneric *fx, int fxsize) {

	vfx.push_back(fx);
	return 0;
}

static bool ignoreSetTime = false;

void GLMain::setTime(float seconds) {

	if (ignoreSetTime || demoTimeSec == seconds)
		return;
	ignoreSetTime = true;

	demoTimeSec = seconds;

	int time = (int) (demoTimeSec * 1000.0);

	// compute options for the new time
	for (unsigned int i=0; i<vfxi.size(); i++) {
		if (vfxi[i]->isActive(time)) {
			for (unsigned int j=0; j < vfxi[i]->opts.size(); j++) {
				vfxi[i]->opts[j]->computeValue(time);	
				vfxi[i]->opts[j]->updateGui();
			}
		}
	}

	// display the new time
	if (timeAdjustment->value != seconds) {
		timeAdjustment->value = seconds;
		gtk_signal_emit_by_name(GTK_OBJECT(timeAdjustment), "value_changed");
	}
	ignoreSetTime = false;
}

void GLMain::setTime(int millisec) {
	setTime(((float)millisec) / 1000.0f );
}	

/******************************************
 *           Callback functions           *
 *****************************************/

/* display call back function. */
gint cbDisplay(GtkWidget *widget, GdkEventExpose *event) {
	glMainptr->reDraw();
	return TRUE;
}

/* init GL callback */
gint cbInit(GtkWidget *widget) {
	if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) { 
		glEnable(GL_DEPTH_TEST);
		glViewport(0,0, widget->allocation.width, widget->allocation.height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0,100, 100,0, -1,1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}

	glMainptr->reDraw();
	return TRUE;
}

// play / stop button call back
gint cbPlayStopButton(GtkWidget *widget, gpointer data) {

	if (glMainptr->playing) {
		// stop playing
		glMainptr->playing = false;

		glMainptr->ctrlDlg.setPlayStopLabel("Play");

		// stop wasting CPU 
		gtk_idle_remove(glMainptr->idleTag);

		// make sure the screen is correctly drawn
		glMainptr->reDraw();
	} else {
		// start playing
		glMainptr->playing = true;
		glMainptr->ctrlDlg.setPlayStopLabel("Stop");
		glMainptr->timeStart = getMilliSec() - (int) (glMainptr->demoTimeSec * 1000.0);
		
		// When idling: play the demo.
		glMainptr->idleTag=gtk_idle_add_priority(GTK_PRIORITY_LOW, cbIdle, glMainptr);
	}
	return TRUE;
}

/* reshape call back function */
gint cbReshape(GtkWidget *widget, GdkEventConfigure *event)
{
	// OpenGL functions can be called only if make_current returns true 
	if (gtk_gl_area_make_current(GTK_GL_AREA(widget))) { 

		int x = widget->allocation.width;
		int y = widget->allocation.height;

		float xy_aspect;

		xy_aspect = (float)x / (float)y;
		glViewport( 0, 0, x, y );

		/* no real 3d
		   glMatrixMode( GL_PROJECTION );
		   glLoadIdentity();
		   glFrustum( -xy_aspect*.08, xy_aspect*.08, -.08, .08, .1, 15.0 );
		 */
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(-0.5,318.5,199.5,-0.5,0,1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		glMainptr->reDraw();
	}
	return TRUE;
}

// idle callback
gint cbIdle(gpointer) {
	glMainptr->reDraw();
	return TRUE;
}

// new effect instance button or effects/new/.. menu
gint cbNewInstance(GtkWidget *widget, gpointer data) {
	FXGeneric *fxcp;

	if (data == 0) {
		// new instance button
		if (glMainptr->selectedFX==0) return TRUE;
		fxcp = glMainptr->selectedFX->create();
	} else {
		// new instance from menu
		fxcp =((FXGeneric *) data)->create();
	}

	FXInstance *fxi = new FXInstance(fxcp);
	glMainptr->addFXInstance(fxi);
	fxi->setStartTime(glMainptr->getTimeMilliSec());
	fxi->setStopTime(glMainptr->getTimeMilliSec() + 10000);

	glMainptr->reDraw();
	return TRUE;
}

/* keyboard callback function */
gint cbKeyboard(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	
	switch(event->keyval) {
		case 27: 
		case 'q':
			exit(0);
			break;

		case ' ': cbPlayStopButton(0, glMainptr);
			  break;
		default:
			glMainptr->reDraw();
	};

	return TRUE;
}

gint cbDemoTime(GtkWidget *widget, gpointer data) {
	gfloat new_time = glMainptr->timeAdjustment->value;

	if (!ignoreSetTime && new_time != glMainptr->getTimeSec()) {
		glMainptr->setTime( new_time );

		// set the timeStart variable, so the current time will change if
		// the demo is playing.
		glMainptr->timeStart = getMilliSec() - glMainptr->getTimeMilliSec();
	}
				
	if (!glMainptr->playing) {
		glMainptr->reDraw();
	}
	return TRUE;
}

// should return FALSE if quit action is canceled
gint cbQuit(GtkWidget *widget, gpointer data) {
	gtk_main_quit();
	return FALSE;
}


void GLMain::addDialogToMenu(GtkWidget *menu, Dlg *dlg) {
	GtkWidget *item = gtk_menu_item_new_with_label(dlg->getName());
	gtk_menu_append(GTK_MENU(menu), item);
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(menuWindowShowHide), dlg);
	gtk_widget_show(item);
}

/********************************************
**    Initialization and GUI drawing       **
********************************************/
	
// open the window, and start.
int GLMain::start(int *argc, char ***argv) {

	glMainptr = this;

	if (started) return 1;
	started=1;

	// intialize GTK
	gtk_set_locale();
	gtk_init(argc, argv);

	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps");
	add_pixmap_directory(PACKAGE_SOURCE_DIR "/pixmaps");

	// Check if OpenGL is supported. 
	if (gdk_gl_query() == FALSE) {
		g_print("OpenGL not supported\n");
		return 0;
	}

	// create the windows
	drawWindow = gtk_window_new( GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(drawWindow), 640, 490);
	gtk_window_set_title(GTK_WINDOW(drawWindow), "Calodox DemoNiac");

	int attrlist[] = {
		GDK_GL_RGBA,
		GDK_GL_DOUBLEBUFFER,
		GDK_GL_NONE
	};           

	// create openGL widget
	glarea = GTK_WIDGET(gtk_gl_area_new(attrlist));

	// Events for widget must be set before X Window is created 
	gtk_widget_set_events(GTK_WIDGET(glarea),
			GDK_EXPOSURE_MASK|
			GDK_BUTTON_PRESS_MASK |
			GDK_BUTTON_RELEASE_MASK |
			GDK_BUTTON_MOTION_MASK
			);

	mc = new MouseController(glarea);

	GtkWidget *menubar;
	GtkItemFactory *menuFactory = setupMenu(drawWindow, &menubar);
	GtkWidget *drawWinVBox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(drawWindow), drawWinVBox);

	gtk_box_pack_start(GTK_BOX(drawWinVBox), menubar, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(drawWinVBox), glarea, TRUE, TRUE, 0);

	gtk_widget_show(menubar);
	gtk_widget_show(drawWinVBox);
	gtk_widget_show(glarea);

	// Redraw image when exposed.  
	gtk_signal_connect(GTK_OBJECT(glarea), "expose_event",
			GTK_SIGNAL_FUNC(cbDisplay), NULL);
	// When window is resized viewport needs to be resized also. 
	gtk_signal_connect(GTK_OBJECT(glarea), "configure_event",
			GTK_SIGNAL_FUNC(cbReshape), NULL);
	// Do initialization when widget has been realized. 
	gtk_signal_connect(GTK_OBJECT(glarea), "realize",
			GTK_SIGNAL_FUNC(cbInit), NULL);
	gtk_signal_connect(GTK_OBJECT(drawWindow), "key_press_event",
			GTK_SIGNAL_FUNC(cbKeyboard), NULL);

	gtk_signal_connect(GTK_OBJECT(drawWindow), "delete_event",
			GTK_SIGNAL_FUNC(cbQuit), NULL);

	timeAdjustment = (GtkAdjustment *) 
		gtk_adjustment_new (0.0, 0.0, MINIMUM_DEMO_DURATION/1000.0f, 1.0, 10.0, 0.0);

	// create the track dialog with the time adjustment
	trackDlg.build();
	trackDlg.show();

	// create the time & effects control dialog
	ctrlDlg.build();
	ctrlDlg.show();

	// create the draw buffer list dialog
	drawBufferList.build();
	drawBufferList.show();
	

	// get the effects/new submenu
	GtkWidget *subMenu = gtk_item_factory_get_widget(menuFactory, "/Effects/New");
	if (subMenu) {
		for (unsigned int j=0; j<vfx.size(); j++) {
		
			GtkWidget *item = gtk_menu_item_new_with_label(vfx[j]->getName());
			gtk_menu_append(GTK_MENU(subMenu), item);
			gtk_signal_connect(GTK_OBJECT(item), "activate",
					GTK_SIGNAL_FUNC(cbNewInstance), vfx[j]);
			gtk_widget_show(item);
		}
	}
	
	// get the windows submenu
	subMenu = gtk_item_factory_get_widget(menuFactory, "/Windows");
	if (subMenu) {
		addDialogToMenu(subMenu, &ctrlDlg);
		addDialogToMenu(subMenu, &trackDlg);
		addDialogToMenu(subMenu, &drawBufferList);
	}

	playing = false;
	selectedFX=vfx[0];


	// Show everything
	gtk_widget_show_all(drawWindow);
	// add the menu
	//  createMenu();

  	// launch the loop
	gtk_main();
	return 0;  
}

/***************************************
**          Main Drawing Method       **
***************************************/

/* let's render all the fx instances */
void GLMain::reDraw() {
	int start, end;
	unsigned int i;


	if (!gtk_gl_area_make_current(GTK_GL_AREA(glarea))) 
		return;

	// get frame start time
	start = getMilliSec();

	if (playing) {
		// update the demo time
		float newTime = ((float)(start - timeStart)) / 1000.0;
		if (newTime >= timeAdjustment->upper) {
			// loop
			newTime = timeAdjustment->lower;
			timeStart = getMilliSec() - (int) (newTime * 1000.0);
		}
		setTime(newTime);		
	}

	for(i = 0; i < vfxi.size(); i++) {
		vfxi[i]->actionAtTime(getTimeMilliSec());
	}

	// display option stuff on screen, like
	// a rectangle
	mc->draw();	

	// work's done, display it
	gtk_gl_area_swapbuffers(GTK_GL_AREA(glarea));

	end = getMilliSec();
	//sprintf(string,"%05.2f", 1000.0/(double)(end - start)); 
        //fps_box->set_text(string);

}


/*
 * adjustDemoDuration(): look when the last effect finishes,
 * and and set the timeAdjustment.
 */
void GLMain::adjustDemoDuration() {
	
	int duration = MINIMUM_DEMO_DURATION;

	std::vector<FXInstance *>::iterator it;

	for (it = vfxi.begin(); it != vfxi.end(); it++) {
		int stop = (*it)->getStopTime();
		if (duration < stop)
			duration = stop;
	}

	float fd = ((float) duration)/1000.0;

	if (timeAdjustment->upper != fd) {
		timeAdjustment->upper = fd;
		gtk_signal_emit_by_name(GTK_OBJECT(timeAdjustment), "changed");
	}
}

/*************************************
**             Saver                **
*************************************/

void GLMain::save(Saver *s) {
    s->saveObjectBegin("DemoNiac");

    if (filename)
    	delete filename;
    filename = new char [strlen(s->getFileName())+1];
    strcpy(filename, s->getFileName());

    s->saveFloat("demoTimeSec", demoTimeSec);

    drawBufferList.save(s);
    
    // now the instances
    for (unsigned int i=0; i<vfxi.size(); i++) {
	vfxi[i]->save(s);
    }

    s->saveObjectEnd();
}

/*************************************
**              Loader              **
*************************************/

void GLMain::load(Loader *l) {

	if (strcmp("DemoNiac", l->nextName())==0) {
		l->nextLine();

		clearAll();

		filename = new char[strlen(l->getFileName())+1];
		strcpy(filename, l->getFileName());

		float time = l->getFloat("demoTimeSec");
		l->nextLine();

		drawBufferList.load(l);

		while (strcmp("FXInstance", l->nextName())==0) {
			addFXInstance(new FXInstance(l, vfx));
		}

		l->checkType(Loader::OBJECT_END);
			
		// compute options for the new time
		demoTimeSec = time +1; // this forces setTime to recompute options
		setTime(time);
	}
}




