/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Time line widget for Gtk.
 * Leto/calodox, 7 2001.
 * (leto@calodox.org)
 */

#ifndef __GTK_TIME_LINE_H__
#define __GTK_TIME_LINE_H__

#include <gdk/gdk.h>
#include <gtk/gtkwidget.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_TIME_LINE(obj)          GTK_CHECK_CAST (obj, gtk_time_line_get_type (), GtkTimeLine)
#define GTK_TIME_LINE_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_time_line_get_type (), GtkTimeLineClass)
#define GTK_IS_TIME_LINE(obj)       GTK_CHECK_TYPE (obj, gtk_time_line_get_type ())


typedef struct _GtkTimeLine        GtkTimeLine;
typedef struct _GtkTimeLineClass   GtkTimeLineClass;

struct _GtkTimeLine
{
  GtkWidget widget;

  /* update policy (GTK_UPDATE_[CONTINUOUS/DELAYED/DISCONTINUOUS]) */
  guint policy : 2;

  /* Button currently pressed or 0 if none */
  guint8 button;

  /* ID of update timer, or 0 if none */
  guint32 timer;

  /* Old values from adjustment stored so we know when something changes */
  gfloat old_value;
  gfloat old_lower;
  gfloat old_upper;

  /* The adjustment object that stores the data for this time_line */
  GtkAdjustment *adjustment;

  /* the linked list for keys */
  GSList *key_list;

  /* when the user grab a key, remember which one */
  GtkAdjustment *key_grab;

  /* old value for key_grab */
  gfloat key_grab_old_value;

  /* viewing limits */
  gfloat lower, upper;

};

struct _GtkTimeLineClass
{
	GtkWidgetClass parent_class;
	void (* key_press) (GtkTimeLine *timeline, GdkEventButton *event, GtkAdjustment *adjustment);
};


GtkWidget*	gtk_time_line_new		(GtkAdjustment *adjustment);
guint		gtk_time_line_get_type		(void);
void 		gtk_time_line_set_update_policy	(GtkTimeLine      *time_line,
						GtkUpdateType  policy);

GtkAdjustment*	gtk_time_line_get_adjustment	(GtkTimeLine      *time_line);
void		gtk_time_line_set_adjustment	(GtkTimeLine      *time_line,
						GtkAdjustment *adjustment);

/*
 * Specify the time range to display. If lower >= upper, 
 * this function will do nothing. 
 */
void 		gtk_time_line_set_range		(GtkTimeLine *time_line, 
						gfloat lower, 
						gfloat upper);

/*
 * Add a key at key->value. The user won't be able to move the key
 * outside adjustment bounds.
 */
void		gtk_time_line_add_key		(GtkTimeLine *time_line, 
						GtkAdjustment *key);

/*
 * remove a key. 
 * if the key is found and removed: return TRUE. Otherwise, return FALSE.
 */
gint 		gtk_time_line_remove_key	(GtkTimeLine *time_line, 
						GtkAdjustment *key); 

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTK_TIME_LINE_H__ */
/* example-end */
