/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * interpolators
 * Leto / calodox, 19.3.2001
 */

#include <stdio.h>
#include "interpolator.h"

FixedInterpolator fixedInterpolator;
LastValueInterpolator lastValueInterpolator;
LinearInterpolator linearInterpolator;
SmoothInterpolator smoothInterpolator;

Interpolator *getInterpByType(int type) {
	switch (type) {
	case 1: return &fixedInterpolator;
	case 2: return &lastValueInterpolator;
	case 3: return &linearInterpolator;
	case 4: return &smoothInterpolator;
	default:
		return &fixedInterpolator;
	}

	fprintf(stderr, "warning: unknown interpolator type, defaulting to Last Value.\n");
	return &lastValueInterpolator;
}

void Interpolator::setValue(int time, OptVal *value, Track *track) {
	Key *k = track->getKeyAt(time);

	if (k) {
		k->value = *value;
	} else {
		track->addKey(time, value);
	}
}

void FixedInterpolator::getValue(int time, Key *key, Track *track) {
	Key *k = track->getKeyAt(0);
	if (k) 
		*track->variable = k->value;
}

void FixedInterpolator::setValue(int time, OptVal *value, Track *track) {
	*track->variable = *value;
	Key * k= track->getKeyAt(0);
	if (k) {
		k->value = *value;
	} else {
		track->addKey(0, value);
	}
}

int FixedInterpolator::getType() {
    return 1;
}

void LinearInterpolator::getValue(int time, Key *key, Track *track) {
		
	//if (track->canInterpolate())
		track->interpolateLinear(getPercent(time, key->time, key->next->time), key);
}

int LinearInterpolator::getType() {
    return 3;
}

void LastValueInterpolator::getValue(int time, Key *key, Track *track) {
	if (key->next->time == time)
		*track->variable = key->next->value;
	else
		*track->variable = key->value;
}

int LastValueInterpolator::getType() {
    return 2;
}

void SmoothInterpolator::getValue(int time, Key *key, Track *track) {
	if (track->canInterpolate())
		track->interpolateSmooth(getPercent(time, key->time, key->next->time), key);
}

int SmoothInterpolator::getType() {
    return 4;
}

