/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * interpolator code for demoNiac, 
 * Leto / calodox with NCT (nct@ysagoon.com) help
 */

#ifndef _INTERPOLATOR_H_
#define _INTERPOLATOR_H_

#include <assert.h>
#include "track.h"

class Interpolator {
public:

	virtual void getValue(int time, Key *key, Track *track)=0;

	virtual void setValue(int time, OptVal *value, Track *track);

	virtual int getType()=0;

	float getPercent(int time, int t1, int t2) { 
		assert(time >= t1 && time <= t2);
		return 1.0f - (((float) (t2-time)) / (float) (t2-t1));
	}
};

class FixedInterpolator : public Interpolator {
	virtual void getValue(int time, Key *key, Track *track);
	virtual void setValue(int time, OptVal *value, Track *track);
	virtual int getType();
};

class LastValueInterpolator : public Interpolator {
	virtual void getValue(int time, Key *key, Track *track);
	virtual int getType();
};
	
class LinearInterpolator : public Interpolator {
	virtual void getValue(int time, Key *key, Track *track);
	virtual int getType();
};

class SmoothInterpolator : public Interpolator {
	void getValue(int time, Key *key, Track *track);
	virtual int getType();
};

	
extern FixedInterpolator fixedInterpolator;
extern LastValueInterpolator lastValueInterpolator;
extern LinearInterpolator linearInterpolator;
extern SmoothInterpolator smoothInterpolator;

Interpolator *getInterpByType(int type);

#endif

