/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * loader.h: Load DemoNiac data from a file
 * leto/calodox, 3.2001
 */

#ifndef _LOADER_H_
#define _LOADER_H_

#include <stdio.h>

#define LOADER_BUFFER_LENGTH 256
class Loader {
	private:
		FILE *in;
		int type;
		char name[128];
		char buffer[LOADER_BUFFER_LENGTH];
		int vlen;
		int line;
		char *filename;

		void nextToken();
		
	public:

		Loader();
		~Loader();

		// return true on success
		bool loadFile(char *fileName);
		char *getFileName() { return filename; }

		// parse the next line.
		// return true if succes, false if error or EOF
		bool nextLine();
		
		enum { NONE, OBJECT, OBJECT_END, STRING, FLOAT, INT, FLOAT_VECT, INT_VECT };
		int nextType() { return type; }

		char *nextName() { return name; }
		
		bool checkType(int type);
		bool checkName(const char *name);
		
		// return the string in the dest buffer
		void getString(char *dest, int maxSize, const char *name = 0);

		float getFloat(const char *name = 0);

		int getInt(const char *name = 0);

		// for a vector, returns it's dimension.
		int getVectLength();

		void getFloatVect(float *vect, const char *name = 0);

		void getIntVect(int *vect, const char *name = 0);

		// to report a load error, call error.
		void error(char *string);
};

#endif
