/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * MouseController.cpp: mouse events managment for
 * Calodox DemoNiac by leto (leto@calodox.org), 7 2001
 */

#include <stdio.h>
#include <time.h>
#include <gtk/gtk.h>
#include "mousecontroller.h"
#include "glmain.h"

// mouse callback
gint cbMouse(GtkWidget *widget, GdkEvent *event, gpointer data) {
	
	MouseController *ctl = (MouseController *) data;

	switch (event->type) { 
	case GDK_BUTTON_PRESS:
		if (event->button.button <= 3 && event->button.button >=1) {
			ctl->buttonState[event->button.button-1] = true;
			ctl->startPos[event->button.button-1][0] = event->button.x;
			ctl->startPos[event->button.button-1][1] = event->button.y;
		}
		if (ctl->listener)
			ctl->listener->clicked(event->button.button - 1, 
					event->button.x, event->button.y, 
					widget->allocation.width,
					widget->allocation.height);
		break;
	case GDK_BUTTON_RELEASE:
		if (event->button.button <= 3 && event->button.button >=1) {
			ctl->buttonState[event->button.button-1] = false;
		}
		break;
	case GDK_MOTION_NOTIFY:
		{
		if (!ctl->listener) 
			return TRUE;

		#ifndef WIN32
		if (gdk_events_pending()) 
			return TRUE;
		#endif

		for (int i=0; i<3; i++) {
			if (ctl->buttonState[i]) {
				ctl->listener->dragged(i, ctl->startPos[i][0], 
						ctl->startPos[i][1],
						event->motion.x,
						event->motion.y,
						widget->allocation.width,
						widget->allocation.height);
				ctl->startPos[i][0] = event->motion.x;
				ctl->startPos[i][1] = event->motion.y;
			}
		}
		}
		break;
	default:
#ifdef DEBUG
		printf("%s:%d: Warning: unknown event type %d\n", __FILE__, __LINE__, event->type);
#endif
		break;
	}

	return TRUE;
}

MouseController::MouseController(GtkWidget *widget) :
	widget(widget)
{
	for (int i=0; i<3; i++) {
		buttonState[i]= false;
	}

	listener = 0;

	gtk_signal_connect(GTK_OBJECT(widget), "button_press_event",
			GTK_SIGNAL_FUNC(cbMouse), this);
	gtk_signal_connect(GTK_OBJECT(widget), "button_release_event",
			GTK_SIGNAL_FUNC(cbMouse), this);
	motionId = gtk_signal_connect(GTK_OBJECT(widget), "motion_notify_event",
			GTK_SIGNAL_FUNC(cbMouse), this);
}

bool MouseController::select(MouseListener *listener) {

	if (this->listener) {
		this->listener->selected(false);
	}

	this->listener = listener;
	listener->selected(true);
	return true;
}

void MouseController::clear() {
	if (listener)
		listener->selected(false);
	listener = 0;
}


// default MouseListener methods
MouseListener::~MouseListener() {
}

void MouseListener::clicked(int button, double x, double y, int width, int height) {
}

void MouseListener::dragged(int button, double xStart, double yStart, 
		double xEnd, double yEnd, int width, int height) {
}

void MouseListener::draw() {
}

void MouseListener::selected(bool state) {
	if (!state)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), false);
}

// general callback for selecting a listener
gint selectMouseListener(GtkWidget *widget, gpointer data) {
	if (GTK_TOGGLE_BUTTON (widget)->active)
		glMainptr->mc->select((MouseListener *) data);
	else 
		glMainptr->mc->clear();
	return TRUE;
}
