/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * 3D Position option for calodox demoniac
 * leto@calodox.org, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "genericoption.h"
#include "opt3dpos.h"
#include "interpolator.h"
#include "glmain.h"

void register3DPosOpt(char *optName, int flags, float *values){
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new Opt3DPos(currentFX, optName, flags, values) );
}

Opt3DPos::Opt3DPos(FXInstance *fxi, char *optName, int flags, float *values) :
		GenericOption(fxi, optName, "3D Position", flags) {

	this->fvalues = values;
	this->value.set(values, 3);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new TrackFloatVect(3, &this->value, &fixedInterpolator);
	}
}

Opt3DPos::~Opt3DPos() {
	if (GTK_TOGGLE_BUTTON (toggle)->active)
		glMainptr->mc->clear();
}

void Opt3DPos::dragged(int button, double xStart, double yStart, 
		double xEnd, double yEnd, int width, int height) {
	float f[3];

	value.getFloat(f, 3);

	switch (button) {
	case 0:
		f[0] += 0.01*(xEnd-xStart);
		f[1] -= 0.01*(yEnd-yStart);
		break;
	case 1:
		f[0] += 0.01*(xEnd-xStart);
		f[2] += 0.01*(yEnd-yStart);
		break;
	}

	value.set(f, 3);

	if (track) {
		track->setValue(glMainptr->getTimeMilliSec(),
				&value);
	}

	value.getFloat(fvalues, 3);

	glMainptr->reDraw();
}

void Opt3DPos::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		value.getFloat(fvalues, 3);
	}
}

void Opt3DPos::draw(GtkBox *box) {

	ignore = false; 

	if ((flags & OPTION_REINIT) == 0) {
		GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

		GtkWidget *t = gtk_toggle_button_new_with_label(name);
		toggle = GTK_TOGGLE_BUTTON(t);
		gtk_box_pack_start(GTK_BOX(hbox), t, TRUE, TRUE, 0);

		gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
				GTK_SIGNAL_FUNC(selectMouseListener), (MouseListener *)this);

		drawInterp(GTK_BOX(hbox));
	}
} 

void Opt3DPos::update() {
	
	/*
	fvalues[0] = trXY->get_x();
	fvalues[1] = trXY->get_y();
	fvalues[2] = trZ->get_z();
	*/
}

void Opt3DPos::updateGui() {
}

void Opt3DPos::save(Saver *s) {
    s->saveObjectBegin("Opt3DPos");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    if (flags & OPTION_REINIT) {
	s->saveFloatVect("fvalues", 3, fvalues);
    } else {
	track->save(s);
    }

    s->saveObjectEnd();
}

void Opt3DPos::load(Loader *l) {
	l->checkName("Opt3DPos");
	l->nextLine();

	name = new char[128];
	l->getString(name, 127, "name");
	l->nextLine();

	flags = l->getInt("flags");
	l->nextLine();

	if (strcmp("fvalues", l->nextName())==0) {
		track = 0;
		l->getFloatVect(fvalues);
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new TrackFloatVect(3, &this->value, &fixedInterpolator);
		track->load(l);
	}

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}


