/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * 3D rotation option for demoniac
 * leto@calodox.org, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "opt3drot.h"
#include "interpolator.h"
#include "glmain.h"

void register3DRotOpt(char *optName, int flags, float value[4][4]){
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new Opt3DRot(currentFX, optName, flags, value) );
}

Opt3DRot::Opt3DRot(FXInstance *fxi, char *optName, int flags, float value[4][4]) :
		GenericOption(fxi, optName, "Rotation", flags) {
	this->fvalue = (float *)value;
	
	// extract the quaternion from the matrix
	Quatern q;
	QFromM4(&q, value);
	this->value.set((float *)&q, 4);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new Track3DRot(&this->value, &fixedInterpolator);
	}
}

Opt3DRot::~Opt3DRot() {
	if (GTK_TOGGLE_BUTTON (toggle)->active)
		glMainptr->mc->clear();
}

void Opt3DRot::dragged(int button, double xStart, double yStart, 
		double xEnd, double yEnd, int width, int height) {

	Vect v;
	Quatern qOrig, qRes;
	value.getFloat((float *) &qOrig, 4);

	switch (button) {
	case 0:
		v.x = 1; v.y = v.z = 0;
		Quatern qx, qy, qTmp;
		QFromRotation(&qx, &v, 0.01*(yStart-yEnd));
		v.x = v.z = 0; v.y = 1;
		QFromRotation(&qy, &v, 0.01*(xStart-xEnd));

		QMul(&qOrig, &qx, &qTmp);
		QMul(&qTmp, &qy, &qRes);

		break;
	case 1:
		Quatern qz;
		v.x= v.y =0; v.z =1;

		QFromRotation(&qz, &v, 0.01*(yEnd-yStart) + 0.01*(xEnd-xStart));
		QMul(&qOrig, &qz, &qRes);
		break;
	default:
		return;
	}

	// save the resulting quaternion
	value.set((float *)&qRes, 4);
	
	// and convert it to a matrix for the effect
	M4FromQ((float (*)[4])fvalue, qRes);
	
	if (track)
		track->setValue(glMainptr->getTimeMilliSec(),
			&value);
	glMainptr->reDraw();
}


void Opt3DRot::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		// convert quaternion to matrix
		Quatern q;
		value.getFloat((float *)&q, 4);
		M4FromQ((float (*)[4])fvalue, q);
	}
}

void Opt3DRot::draw(GtkBox *box) {
	ignore = false; 

	if ((flags & OPTION_REINIT) == 0) {
		GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

		GtkWidget *t = gtk_toggle_button_new_with_label(name);
		toggle = GTK_TOGGLE_BUTTON(t);
		gtk_box_pack_start(GTK_BOX(hbox), t, TRUE, TRUE, 0);

		gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
				GTK_SIGNAL_FUNC(selectMouseListener), (MouseListener *)this);

		drawInterp(GTK_BOX(hbox));
	}
} 

void Opt3DRot::update() {
	//rot->get_float_array_val(fvalue);
}

void Opt3DRot::updateGui() {
	//rot->set_float_array_val(fvalue);
}

void Opt3DRot::save(Saver *s) {
	s->saveObjectBegin("Opt3DRot");

	s->saveString("name", name);
	s->saveInt("flags", flags);

	if (flags & OPTION_REINIT) {
		float f[4];
		value.getFloat(f, 4);
		s->saveFloatVect("fvalue", 4, f);
	} else {
		track->save(s);
	}

	s->saveObjectEnd();
}

void Opt3DRot::load(Loader *l) {
	assert(strcmp(l->nextName(), "Opt3DRot")==0);
	l->nextLine();

	name = new char[128];
	l->getString(name, 127);
	l->nextLine();

	flags = l->getInt();
	l->nextLine();

	if (strcmp("fvalue", l->nextName())==0) {
		track = 0;
		Quatern q;
		l->getFloatVect((float *)&q);
		M4FromQ((float (*)[4]) fvalue, q );
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new Track3DRot(&this->value, &fixedInterpolator);
		track->load(l);
	}

	assert(l->nextType() == Loader::OBJECT_END);
	l->nextLine();
}


