/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Color option for demoniac
 * leto@calodox.org, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "optcolor.h"
#include "interpolator.h"
#include "glmain.h"
#include "report.h"

void registerColorOpt(char *optName, int flags, float *values) {
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptColor(currentFX, optName, flags, values));
}

OptColor::OptColor(FXInstance *fxi, char *optName, int flags, float *values) :
		GenericOption(fxi, optName, "Color", flags),
		fvalues(values),
		dialog(0)
{

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new TrackFloatVect(4, &this->value, &fixedInterpolator);
	}
	value.set(values, 4);
}

gint cbOptColor(GtkWidget *widget, gpointer data) {
	OptColor *obj = (OptColor *) data;

	if (obj->ignore)
		return TRUE;

	debug("cb color");

	obj->update();	
	obj->value.set(obj->fvalues, 4);

	if (obj->track) {
		obj->track->setValue(glMainptr->getTimeMilliSec(), 
				&obj->value);
	}

	glMainptr->reDraw();

	return TRUE;
}

void OptColor::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		value.getFloat(fvalues, 4);
	}
}

gint cbOptColorButton(GtkWidget *widget, gpointer data) {
	OptColor *obj = (OptColor *) data;

	gtk_widget_show(obj->dialog);
	obj->updateGui();
		
	return TRUE;
}

gint cbOptColorClose(GtkWidget *widget, gpointer data) {
	OptColor *obj = (OptColor *) data;

	gtk_widget_hide(obj->dialog);
		
	return TRUE;
}

void OptColor::draw(GtkBox *box) {
	ignore = false;

	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

	GtkWidget *button = gtk_button_new_with_label(name);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(cbOptColorButton), this);
	
	dialog = gtk_color_selection_dialog_new(name);
	GtkWidget * colorSel = GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel;
	gtk_color_selection_set_opacity(GTK_COLOR_SELECTION(colorSel), TRUE);

	if ((flags & OPTION_REINIT) == 0) {

		gtk_signal_connect(GTK_OBJECT(
			GTK_COLOR_SELECTION_DIALOG(dialog)->ok_button),
			"clicked",
			(GtkSignalFunc)(cbOptColorClose), this);

		gtk_signal_connect(GTK_OBJECT(
			GTK_COLOR_SELECTION_DIALOG(dialog)->cancel_button),
			"clicked",
			GTK_SIGNAL_FUNC(cbOptColorClose), this);
		
		gtk_widget_hide(GTK_COLOR_SELECTION_DIALOG(dialog)->help_button);

		gtk_signal_connect(GTK_OBJECT(colorSel), "color_changed",
		             (GtkSignalFunc) cbOptColor, (gpointer)this);

		drawInterp(GTK_BOX(hbox));
	}
	updateGui();	
}

void OptColor::update() {
	gdouble db[4];

	gtk_color_selection_get_color(GTK_COLOR_SELECTION(
		GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel),
		db);

	for (int i=0; i<4; i++) {
		fvalues[i] = (float) db[i];
	}
}

void OptColor::updateGui() {

	if (dialog == 0) 
		return;

	GtkArg arg;

	// updating the GUI is only usefull if the color
	// dialog box is visible
	arg.name = "visible";
	gtk_widget_get(dialog, &arg);
	if (!GTK_VALUE_BOOL(arg)) return;

	gdouble db[4];

	for(int i=0; i<4; i++) {
		db[i] = (gdouble) fvalues[i];
		if (db[i] < 0 || db[i] > 1) {
			debug("Warning: invalid color value !");
			db[i]= (i== 3 ? 1 : 0);
		}
	}
	
	ignore = true;
	gtk_color_selection_set_color(GTK_COLOR_SELECTION(
		GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel),
		db);
	ignore = false;
}

void OptColor::save(Saver *s) {
    s->saveObjectBegin("OptColor");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    if (flags & OPTION_REINIT) {
	s->saveFloatVect("color", 8, fvalues);
    } else {
	track->save(s);
    }

    s->saveObjectEnd();
}

void OptColor::load(Loader *l) {

	l->checkName("OptColor");
	l->nextLine();

	name = new char[128];
	l->getString(name, 127, "name");
	l->nextLine();

	flags = l->getInt("flags");
	l->nextLine();

	if (strcmp("color", l->nextName())==0) {
		track = 0;
		l->getFloatVect(fvalues);
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new TrackFloatVect(4, &this->value, &fixedInterpolator);
		track->load(l);
	}

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}


