/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * DrawBuffer option for demoniac
 * leto@calodox.org, 9.2001
 */

#include <fx.h>
#include "loader.h"
#include "saver.h"
#include "genericoption.h"
#include "optdrawbuff.h"
#include "glmain.h"
#include "interpolator.h"
#include "report.h"


void registerDrawBufferOpt(char *optName, int flags, DrawBuffer **buffer) {
	assert(buffer != 0);
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptDrawBuffer(currentFX, optName, flags, buffer) );
}

OptDrawBuffer::OptDrawBuffer(FXInstance *fxi, char *optName, int flags, DrawBuffer **buffer) :
	GenericOption(fxi, optName, "Draw buffer", flags),
	buffer(buffer)
{
	DrawBuffer *defBuff= *buffer;

	if (defBuff) {
		*buffer = glMainptr->drawBufferList.addDrawBuffer(defBuff->name, defBuff->x, defBuff->y);
		(*buffer)->buff = defBuff->buff;
		(*buffer)->data = defBuff->data;
	} else {
		*buffer = glMainptr->drawBufferList.addDrawBuffer(0,0,0);
	}

	value.set((char *)(*buffer)->name);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new Track(&this->value, &fixedInterpolator);
	}
}

void OptDrawBuffer::cbClicked(GtkWidget *widget, OptDrawBuffer *obj) {
	glMainptr->drawBufferList.watchOptDrawBuffer(obj);	
}

void OptDrawBuffer::changeDrawBuffer(DrawBuffer *buff) {

	if (buff != (*buffer) && buff!=0) {
		*buffer = buff;
		value.set((char *)buff->name);
		updateGui();
	}
}

// draw a button wich calls drawBufferList
void OptDrawBuffer::draw(GtkBox *box) {

	// create an hbox with option's name label packed
        GtkBox *hbox = drawLabel(box);

        button = gtk_button_new();

	assert(*buffer != 0);
	label = gtk_label_new((*buffer)->name);
	gtk_container_add(GTK_CONTAINER(button), label);

        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0); 
                                
        if ((flags & OPTION_REINIT) == 0) {
                gtk_signal_connect(GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(cbClicked), this);                                                          
                drawInterp(GTK_BOX(hbox));                                                                          
        }
        updateGui();
}


void OptDrawBuffer::update() {
	*buffer = glMainptr->drawBufferList.query(value.asString());
}

void OptDrawBuffer::updateGui() {
	gtk_label_set_text(GTK_LABEL(label), (*buffer)->name);
}

void OptDrawBuffer::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		update();
	}
}

void OptDrawBuffer::save(Saver *s) {

	s->saveObjectBegin("OptDrawBuffer");

	s->saveString("name", name);
	s->saveInt("flags", flags);

	if (flags & OPTION_REINIT) {
		s->saveString("value", (*buffer)->name);
		s->saveInt("Xsize", (*buffer)->x);
		s->saveInt("Ysize", (*buffer)->y);
	} else {
		track->save(s);
	}

	s->saveObjectEnd();
}

void OptDrawBuffer::load(Loader *l) {

	if (!l->checkType(Loader::OBJECT) || !l->checkName("OptDrawBuffer"))
		return;

	l->nextLine();

        name = new char[128];
        l->getString(name, 127, "name");
        l->nextLine();

        flags = l->getInt("flags");
        l->nextLine();

        if (strcmp("value", l->nextName())==0) {
                track = 0;
		char tmp[128];
		l->getString(tmp, sizeof(tmp));
                l->nextLine();

		int x = l->getInt("Xsize");
                l->nextLine();
		int y = l->getInt("Ysize");
                l->nextLine();

		*buffer = glMainptr->drawBufferList.addDrawBuffer(tmp, x, y);
        }
        
        if (l->nextType() == Loader::OBJECT) {
                track = new Track(&this->value, &fixedInterpolator);
                track->load(l);
        }
        
        l->checkType(Loader::OBJECT_END);
        l->nextLine();
}

