/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Integer option for demoniac
 * leto@calodox.org, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "optint.h"
#include "interpolator.h"
#include "glmain.h"
#include "report.h"

void registerIntOpt(char *optName, int flags, int *value,
                int min, int max) {

	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptInt(currentFX, optName, flags, value, min, max) );
}

OptInt::OptInt(FXInstance *fxi, char *optName, int flags, int *value,
                int min, int max ) :
		GenericOption(fxi, optName, "Number", flags) {

        this->ivalue = value;
	this->value.set(value);
        this->min = min;
        this->max = max;
                
	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new TrackIntVect(1, &this->value, &fixedInterpolator);
	}
}                                  
                                   
gint cbOptInt(GtkWidget *widget, gpointer data) {

	OptInt *obj = (OptInt *) data;

	if (obj->ignore) return TRUE;

	debug("cb int");

	*obj->ivalue = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(obj->spin)); 
	obj->value.set(obj->ivalue);	
	if (obj->track) {
		obj->track->setValue(glMainptr->getTimeMilliSec(), 
				&obj->value);
	}

	glMainptr->reDraw();

	return TRUE;
}

void OptInt::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		*ivalue = value.asInt();
	}
}

void OptInt::draw(GtkBox *box) {

	GtkBox *hbox = drawLabel(box);
	
	// spin button
	GtkAdjustment *adj = (GtkAdjustment *) gtk_adjustment_new(
		(gfloat) value.asFloat(), (gfloat)min, (gfloat)max, 1, 10, 1);
	spin = gtk_spin_button_new(adj, 1, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 5);
	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), flags & OPTION_LIMIT_WRAP);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);

	if ((flags & OPTION_REINIT) == 0) {
		gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
			GTK_SIGNAL_FUNC(cbOptInt), this);
		drawInterp(GTK_BOX(hbox));
	}

	updateGui();
}         

void OptInt::update() {
	*ivalue = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
}

void OptInt::updateGui() {
	ignore = true;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), (gfloat)*ivalue);
	ignore = false;
}

void OptInt::save(Saver *s) {
    s->saveObjectBegin("OptInt");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    if (flags & OPTION_REINIT) {
	s->saveInt("ivalue", *ivalue);
    } else {
	track->save(s);
    }

    s->saveObjectEnd();
}

void OptInt::load(Loader *l) {
	assert(strcmp(l->nextName(), "OptInt")==0);
	l->nextLine();

	name = new char[128];
	l->getString(name, 127);
	l->nextLine();

	flags = l->getInt();
	l->nextLine();

	if (strcmp("ivalue", l->nextName())==0) {
		track = 0;
		*ivalue = l->getInt();
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new TrackIntVect(1, &this->value, &fixedInterpolator);
		track->load(l);
	}

	assert(l->nextType() == Loader::OBJECT_END);
	l->nextLine();
}


