/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Screen position option for calodox demoniac
 * leto@calodox.org, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "genericoption.h"
#include "optscrpos.h"
#include "interpolator.h"
#include "mousecontroller.h"
#include "glmain.h"

void registerScreenPosOpt(char *optName, int flags, float *values){
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptScreenPos(currentFX, optName, flags, values) );
}

OptScreenPos::OptScreenPos(FXInstance *fxi, char *optName, int flags, float *values) :
		GenericOption(fxi, optName, "Screen Position", flags) {

	this->fvalues = values;
	this->value.set(values, 2);

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new TrackFloatVect(2, &this->value, &fixedInterpolator);
	}
}

OptScreenPos::~OptScreenPos() {
	if (GTK_TOGGLE_BUTTON (toggle)->active)
		glMainptr->mc->clear();
}

void OptScreenPos::clicked(int button, double x, double y, int width, int height) {

	if (button == 0) {
		fvalues[0] = (float) (x/ (double) width);
		fvalues[1] = (float) (y/ (double) height);
	}

	value.set(fvalues, 2);

	if (track) {
		track->setValue(glMainptr->getTimeMilliSec(),
				&value);
	}

	glMainptr->reDraw();
}

void OptScreenPos::dragged(int button, double xStart, double yStart, 
		double xEnd, double yEnd, int width, int height) {
	clicked(button, xEnd, yEnd, width, height);
}

void OptScreenPos::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		value.getFloat(fvalues, 2);
	}
}

void OptScreenPos::draw(GtkBox *box) {

	ignore = false; 

	if ((flags & OPTION_REINIT) == 0) {
		GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

		GtkWidget *t = gtk_toggle_button_new_with_label(name);
		toggle = GTK_TOGGLE_BUTTON(t);
		gtk_box_pack_start(GTK_BOX(hbox), t, TRUE, TRUE, 0);

		gtk_signal_connect(GTK_OBJECT(toggle), "toggled",
				GTK_SIGNAL_FUNC(selectMouseListener), (MouseListener *)this);

		drawInterp(GTK_BOX(hbox));
	}
} 

void OptScreenPos::update() {
}

void OptScreenPos::updateGui() {
}

void OptScreenPos::save(Saver *s) {
    s->saveObjectBegin("OptScreenPos");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    s->saveFloatVect("fvalues", 2, fvalues);

    if ((flags & OPTION_REINIT) == 0) {
	track->save(s);
    }

    s->saveObjectEnd();
}

void OptScreenPos::load(Loader *l) {
	l->checkName("OptScreenPos");
	l->nextLine();

	name = new char[128];
	l->getString(name, 127, "name");
	l->nextLine();

	flags = l->getInt("flags");
	l->nextLine();

	if (strcmp("fvalues", l->nextName())==0) {
		track = 0;
		l->getFloatVect(fvalues);
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new TrackFloatVect(2, &this->value, &fixedInterpolator);
		track->load(l);
	}

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}


