/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * String option for demoniac
 * leto@calodox.org, 2001
 */

#include <fx.h>
#include "fxinstance.h"
#include "optstring.h"
#include "interpolator.h"
#include "glmain.h"
#include "report.h"

void registerStringOpt(char *optName, int flags, char *string, int maxLen) {
	if (currentFX)
		currentFX->opts.push_back( (GenericOption *)
			new OptString(currentFX, optName, flags, string, maxLen) );
}

OptString::OptString(FXInstance *fxi, char *name, int flags, char *value, int maxlen) :
		GenericOption(fxi, name, "String", flags) {
	this->svalue = value;
	this->value.set(value);
	this->len = maxlen;

	if (flags & OPTION_REINIT) {
		track = 0;
	} else {
		track = new Track(&this->value, &fixedInterpolator);
	}
}

gint cbOptString(GtkWidget *widget, gpointer data) {
	OptString *obj = (OptString *) data;

	if (obj->ignore) return TRUE;

	debug("cb string");

	obj->update();
	obj->value.clear();
	obj->value.set(obj->svalue);

	obj->track->setValue(glMainptr->getTimeMilliSec(), &obj->value);
	
	return TRUE;
}

void OptString::computeValue(int time) {
	if (track) {
		track->computeValue(time);
		strncpy(svalue, value.asString(), len);
		svalue[len-1] = 0;
	}
}

void OptString::draw(GtkBox *box) {
	GtkBox *hbox = drawLabel(box);

	entry = gtk_entry_new();
	gtk_box_pack_start(hbox, entry, TRUE, TRUE, 5);
	updateGui();

	if ((flags & OPTION_REINIT) == 0) {
		drawInterp(hbox);
		gtk_signal_connect(GTK_OBJECT(entry), "changed",
				(GtkSignalFunc)(cbOptString), this);
	}
} 

void OptString::update() {
	strncpy(svalue, gtk_entry_get_text(
		GTK_ENTRY(entry)), len-1);
	svalue[len-1] = 0;
}

void OptString::updateGui() {
	ignore = true;
	gtk_entry_set_text(GTK_ENTRY(entry), svalue);
	ignore = false;
}

void OptString::save(Saver *s) {
    s->saveObjectBegin("OptString");

    s->saveString("name", name);
    s->saveInt("flags", flags);

    if (flags & OPTION_REINIT) {
	s->saveString("svalue", svalue);
    } else {
	track->save(s);
    }

    s->saveObjectEnd();
}



void OptString::load(Loader *l) {
	l->checkName("OptString");
	l->nextLine();

	name = new char[128];
	l->getString(name, 127);
	l->nextLine();

	flags = l->getInt("flags");
	l->nextLine();

	if (strcmp("svalue", l->nextName())==0) {
		track = 0;
		svalue = new char[128];
		l->getString(svalue, 127);
		l->nextLine();
	} 

	if (l->nextType() == Loader::OBJECT) {
		track = new Track(&this->value, &fixedInterpolator);
		track->load(l);
	}

	l->checkType(Loader::OBJECT_END);
	l->nextLine();
}
