/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * OptVal: class to contain & manage a value of
 * any type for options.
 * leto@calodox.org, 8.2001
 */

#ifndef _OPTVAL_H_
#define _OPTVAL_H_

#include "saver.h"
#include "loader.h"

#define OPTVAL_MAX_INT 4
#define OPTVAL_MAX_FLOAT 16

class OptVal {
private:
	typedef enum { NONE, INT, FLOAT, BOOL, STRING } Type;
	Type type;

	typedef union
	{
		int iv[OPTVAL_MAX_INT];
		float fv[OPTVAL_MAX_FLOAT];
		bool b;
		char *s;
	} ValueUnion;

	ValueUnion val;


	// size of float & int vectors
	int size;

public:
	OptVal();
	~OptVal();

	void clear();

	void set(int *i, int dim=1);
	void set(float *f, int dim=1);
	void set(bool b);
	void set(char *s);

	int asInt();
	void getInt(int *dest, int dim);
	
	float asFloat();
	void getFloat(float *dest, int dim);

	bool asBool();

	char *asString();

	
	void save(Saver *s);
	void load(Loader *l);

	// take care of copying string
	OptVal &operator = (OptVal & src);
};

#endif //_OPTVAL_H_
