/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Saver: tool to save objects in a file.
 * leto/calodox, 3.2001
 */

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include "saver.h"

Saver::Saver() {
    out = 0;
    lev = 0;
    filename = 0;
}

Saver::~Saver() {
    if (filename)
    	delete filename;

    if (out)
	fclose(out);
}

bool Saver::saveFile(char *name) {

	out = fopen(name, "wt");

	if (filename)
		delete filename;
	filename = new char[strlen(name)+1];
	strcpy(filename, name);

	if (out) return true;
	else return false;
}

void Saver::indent() {
    for (int i=0; i<lev; i++) {
	fputc('\t', out);
    }
}

void Saver::saveObjectBegin(char *name) {

    indent();
    fprintf(out, "object \"%s\" {\n", name);
    lev++;
}

void Saver::saveObjectEnd() {
    lev--;
    indent();
    fprintf(out, "}\n\n");
}

void Saver::saveString(char *name, const char *str) {
    indent();
    fprintf(out, "string \"%s\" \"%s\"\n", name, str);
}

void Saver::saveFloat(char *name, float f) {
    indent();
    fprintf(out, "float \"%s\" %f\n", name, f);
}

void Saver::saveInt(char *name, int i) {
    indent();
    fprintf(out, "int \"%s\" %d\n", name, i);
}

void Saver::saveFloatVect(char *name, int size, float *vect) {
	assert(size);
	indent();
	fprintf(out, "fvect \"%s\" %d ", name, size);
	for (int i=0; i<size; i++)
	    fprintf(out, " %f", vect[i]);
	fprintf(out, "\n");
}

void Saver::saveIntVect(char *name, int size, int *vect) {
	indent();
	fprintf(out, "ivect \"%s\" %d\n", name, size);
	for (int i=0; i<size; i++)
	    fprintf(out, " %d", vect[i]);
}


