/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * timer.c: a small, portable function
 * that returns time in milliseconds
 */

#include "timer.h"

#ifdef WIN32

#include <windows.h>

int getMilliSec() {
	return GetTickCount();
}

#else

#include <sys/time.h>

int getMilliSec() {
        static int initialized = 0;
        static struct timeval start;
        struct timeval now;

        if (!initialized) {
                initialized = 1;
                gettimeofday(&start, 0);
        }

        gettimeofday(&now, 0);

        return (now.tv_sec-start.tv_sec)*1000+(now.tv_usec-start.tv_usec)/1000;
}

#endif

