/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Key track dialog window for calodox DemoNiac
 * leto@calodox.org, 8.2001
 */

#include <gtk/gtk.h>
#include "gtktimeline.h"
#include "glmain.h"
#include "trackdlg.h"
#include "genericoption.h"
#include "interpolator.h"
#include "report.h"

#define EVENT_METHOD(i, x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x

static GtkAdjustment *currentKey;

TrackDlg::TrackDlg() : pixSize(500) {
}

TrackDlg::~TrackDlg() {
}

void TrackDlg::build() {

	// create the window
	window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_window_set_default_size(GTK_WINDOW(window), 640, 160);
	gtk_window_set_title(GTK_WINDOW(window), "Demoniac: key editor");
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(Dlg::cbDeleteWindow), (Dlg *) this);
	// make sure to get motion_notify events
	gtk_widget_set_events( window, GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK );

	// create the scrolled window
	GtkWidget *scrWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrWin),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(window), scrWin);

	// create a vertical box 
	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrWin), vbox);

	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

	// create the zoom in / zoom out buttons
	GtkWidget *zoomIn = gtk_button_new_with_label("Zoom In");
	gtk_signal_connect(GTK_OBJECT(zoomIn), "clicked",
			GTK_SIGNAL_FUNC(cbZoomIn), this);
	gtk_box_pack_start(GTK_BOX(hbox), zoomIn, TRUE, TRUE, 0);
	GtkWidget *zoomOut = gtk_button_new_with_label("Zoom Out");
	gtk_signal_connect(GTK_OBJECT(zoomOut), "clicked",
			GTK_SIGNAL_FUNC(cbZoomOut), this);
	gtk_box_pack_start(GTK_BOX(hbox), zoomOut, TRUE, TRUE, 0);

	// get time adjustment
	GtkAdjustment *adj = glMainptr->getTimeAdjustment();
	assert(adj != 0);

	// create an horizontal ruler
	ruler = gtk_hruler_new();
	gtk_ruler_set_range( GTK_RULER(ruler), 0, adj->upper, 0, adj->upper);
	gtk_signal_connect(GTK_OBJECT(adj), "changed", 
			GTK_SIGNAL_FUNC(cbTimeChanged), this);

	gtk_widget_set_usize( ruler, pixSize, -2);
	gtk_box_pack_end(GTK_BOX(hbox), ruler, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);


	// create the tree widget
	root = GTK_TREE(gtk_tree_new());

	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(root), TRUE, TRUE, 0);

	// create the scrollbar for time range control.
	// by default, zoom out to display everything.
	timeRange = GTK_ADJUSTMENT(gtk_adjustment_new(adj->value, 0, adj->upper, 
			adj->step_increment, adj->page_increment, adj->upper));
	gtk_signal_connect(GTK_OBJECT(timeRange), "changed",
			GTK_SIGNAL_FUNC(cbZoom), this);
	gtk_signal_connect(GTK_OBJECT(timeRange), "value_changed",
			GTK_SIGNAL_FUNC(cbZoom), this);
	// create the horizontal scroll bar for time
	GtkWidget *scroll = gtk_hscrollbar_new(GTK_ADJUSTMENT(timeRange));
	gtk_widget_set_usize( scroll, pixSize, -2);
	hbox = gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), scroll, FALSE, FALSE, 0);

	gtk_widget_show_all(scrWin);

}

const char *TrackDlg::getName() {
	return "Key editor";
}

void TrackDlg::addFXInstance(FXInstance *fxi) {
	if (fxi->treeItem)
		return;

	// create a new item
	fxi->treeItem = gtk_tree_item_new_with_label(fxi->name);

	// create a subtree
	GtkWidget *tree = gtk_tree_new();
	gtk_tree_append(root, fxi->treeItem);
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(fxi->treeItem), tree);
	gtk_widget_show(fxi->treeItem);
	gtk_widget_show(tree);
}

void TrackDlg::removeFXInstance(FXInstance *fxi) {
	std::vector<GenericOption *>::iterator it;

	for (it = fxi->opts.begin(); it != fxi->opts.end(); it++)
		removeOption(*it);

	gtk_tree_remove_item(root, fxi->treeItem);
	fxi->treeItem=0;
}

void TrackDlg::addOption(GenericOption *opt) {


	Track *track = opt->track;

	if (track == 0)
		return;

	if (track->interpolator->getType() == 1 || track->treeItem !=0)
		return;

	GtkWidget *tree= GTK_TREE_ITEM(opt->owner->treeItem)->subtree;
	if (tree == 0) {
		tree = gtk_tree_new();
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(opt->owner->treeItem), tree);
	}

	track->treeItem = gtk_tree_item_new();

	// create the hbox
	GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(track->treeItem), hbox);

	// the label
	GtkWidget *label = gtk_label_new(opt->name);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, FALSE, 0);

	// the timeline

	// get time adjustment
	GtkAdjustment *adj = glMainptr->getTimeAdjustment();

	if (adj == 0)
		adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 10, 1, 10, 0));

	assert(track->timeline == 0);
	track->timeline = gtk_time_line_new(adj);
	gtk_widget_set_usize(track->timeline, pixSize, -2);
	gtk_box_pack_end(GTK_BOX(hbox), track->timeline, FALSE, FALSE, 0);

	gtk_signal_connect_object( GTK_OBJECT(track->timeline), "motion_notify_event",
			(GtkSignalFunc)EVENT_METHOD(ruler, motion_notify_event),
			GTK_OBJECT(ruler) );
	
	gtk_signal_connect(GTK_OBJECT(track->timeline), "key_press",
			GTK_SIGNAL_FUNC(cbKeyPress), track);

	gtk_signal_connect(GTK_OBJECT(timeRange), "changed",
			GTK_SIGNAL_FUNC(cbZoomTimeLine), track->timeline);
	
	gtk_signal_connect(GTK_OBJECT(timeRange), "value_changed",
			GTK_SIGNAL_FUNC(cbZoomTimeLine), track->timeline);
	cbZoomTimeLine(timeRange, GTK_TIME_LINE(track->timeline));

	// now add the new treeItem to the effect branch
	gtk_tree_append(GTK_TREE(tree), track->treeItem);

	// add all the keys 
	for (Key *k = opt->track->first; k != 0; k=k->next) {
		addKey(opt->track, k);
	}

	gtk_widget_show_all(GTK_WIDGET(track->treeItem));

}

void TrackDlg::removeOption(GenericOption *opt) {
	if (opt == 0 || opt->track == 0 || opt->track->treeItem == 0)
		return;


	for (Key *k = opt->track->first; k != 0; k=k->next) {
		removeKey(opt->track, k);
	}
		
	gtk_signal_disconnect_by_data(GTK_OBJECT(timeRange), opt->track->timeline);

	// remove option's branch of the tree
	GtkWidget *tree= GTK_TREE_ITEM(opt->owner->treeItem)->subtree;

	GList l;
	l.data = (gpointer) opt->track->treeItem;
	l.next = 0;

	gtk_tree_remove_items(GTK_TREE(tree), &l);
	opt->track->treeItem = 0;
	opt->track->timeline = 0;
}

void TrackDlg::addKey(Track *track, Key *key) {

	if (track->treeItem == 0 || track->timeline == 0)
		return;

	assert (key->adj == 0); 

	debug("added key");

	key->adj = GTK_ADJUSTMENT(gtk_adjustment_new(((float)key->time)/1000.0, 0, FLT_MAX, 1, 2, 3));
	gtk_signal_connect(GTK_OBJECT(key->adj), "value_changed", 
			GTK_SIGNAL_FUNC(cbKeyMoved), key);

	gtk_time_line_add_key(GTK_TIME_LINE(track->timeline), key->adj);


}

void TrackDlg::removeKey(Track *track, Key *key) {
	if (track->treeItem == 0 || track->timeline == 0 )
		return;

	assert(key->adj != 0);

	debug("removed key");

	gtk_time_line_remove_key(GTK_TIME_LINE(track->timeline), key->adj);

	gtk_object_unref(GTK_OBJECT(key->adj));

	key->adj = 0;
}

void TrackDlg::moveKey(Key *key) {
	
	if (key->adj == 0)
		return;

	if ((int) (key->adj->value * 1000.0) != key->time) {

		// update the adjustment, and emit signal
		key->adj->value = ((float) key->time) / 1000.0;

		gtk_signal_emit_by_name(GTK_OBJECT(key->adj), "value_changed");
	}
}

void TrackDlg::cbKeyMoved(GtkAdjustment *adj, gpointer data) {
	debug("key moved");

	Key *k = (Key *) data;

	assert(k->owner);

	gtk_signal_handler_block_by_data(GTK_OBJECT(adj), data);

	k->owner->moveKey(k, (int) (adj->value * 1000.0));

	gtk_signal_handler_unblock_by_data(GTK_OBJECT(adj), data);

}

void TrackDlg::cbKeyPress(GtkTimeLine *timeline, GdkEventButton *event, GtkAdjustment *adj, gpointer data) {
	Track *track = (Track *) data;
	
	// now create the popup menu
	GtkWidget *contextMenu = gtk_menu_new();
	
	GtkWidget *item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(contextMenu), item);
	gtk_signal_connect(GTK_OBJECT(item), "activate",
			GTK_SIGNAL_FUNC(cbRemoveKey), track);

	gtk_widget_show_all(contextMenu);

	currentKey = adj;
	gtk_menu_popup(GTK_MENU(contextMenu), 0, 0, 0, 0, 3, event->time);

	gtk_widget_unref(contextMenu);
	debug("a key was pressed.");
}

void TrackDlg::cbRemoveKey(GtkTimeLine *timeline, gpointer data) {
	Track *track = (Track *) data;

	track->removeKey((int) (currentKey->value * 1000.0));
	
	debug("cbRemoveKey");
	
}

void TrackDlg::cbTimeChanged(GtkAdjustment *adj, TrackDlg *obj) {
	gtk_ruler_set_range( GTK_RULER(obj->ruler), 0, adj->upper, 0, adj->upper);
}

void TrackDlg::cbZoom(GtkAdjustment *adj, TrackDlg *obj) {

	gtk_ruler_set_range( GTK_RULER(obj->ruler),
			adj->value, 
			adj->value + adj->page_size,
			adj->value, adj->upper);
}

void TrackDlg::cbZoomTimeLine(GtkAdjustment *adj, GtkTimeLine *timeLine) {
	gtk_time_line_set_range(timeLine, adj->value, adj->value + adj->page_size);
}

void TrackDlg::cbZoomIn(GtkWidget *widget, TrackDlg *obj) {

	obj->timeRange->page_size /= 2.0f;
	gtk_signal_emit_by_name(GTK_OBJECT(obj->timeRange), "changed");
}

void TrackDlg::cbZoomOut(GtkWidget *widget, TrackDlg *obj) {

	obj->timeRange->page_size = obj->timeRange->page_size * 2.0f > obj->timeRange->upper ?
		obj->timeRange->upper : obj->timeRange->page_size * 2.0f;
	gtk_signal_emit_by_name(GTK_OBJECT(obj->timeRange), "changed");
}
			

