/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
	matvect.h: vector + matrix operations.
	Leto / Calodox, 1998.
*/

#include <math.h>

#include "matvect.h"
#include <string.h>
#include <stdio.h>

/* vector operations: */

void VAdd( Vect *v1, Vect *v2, Vect *res ) {
    res->x = v1->x + v2->x;
    res->y = v1->y + v2->y;
    res->z = v1->z + v2->z;
}	
    
void VSub( Vect *v1, Vect *v2, Vect *res ) {
    res->x = v1->x - v2->x;
    res->y = v1->y - v2->y;
    res->z = v1->z - v2->z;
}	

void VMul( float fact, Vect *v ) {
    v->x *= fact;
    v->y *= fact;
    v->z *= fact;
}

float VScalar( Vect *v1, Vect *v2 ) {
    return v1->x * v2->x + 
    	   v1->y * v2->y +
	   v1->z * v2->z;
}

void VCross( Vect *v1, Vect *v2, Vect *res ) {
   res->x = v1->y * v2->z - v1->z * v2->y;
   res->y = v1->z * v2->x - v1->x * v2->z;
   res->z = v1->x * v2->y - v1->y * v2->x;
}

void VNormalize( Vect *v1 ) {
    float f = 1/sqrtf(v1->x*v1->x + v1->y*v1->y + v1->z*v1->z);
    //if (f < 0.01) return;
    //f=1/f;
    v1->x *= f;
    v1->y *= f;
    v1->z *= f;
}

float VLength( Vect *v1 ) {
    float f = (float) sqrt(v1->x*v1->x + v1->y*v1->y + v1->z*v1->z);
	return f;
}
    
void VInvert( Vect *v ) {
    v->x = -v->x;
    v->y = -v->y;
    v->z = -v->z;
}

/* matrix operations: */

void MIdent( Matrix m ) {
    memset( m, 0, sizeof(Matrix));
    m[0][0] = 1;
    m[1][1] = 1;
    m[2][2] = 1;
}

void M4Ident( Matrix4 m ) {
    memset( m, 0, sizeof(Matrix4));
    m[0][0] = 1;
    m[1][1] = 1;
    m[2][2] = 1;
	m[3][3] = 1;
}

/* m1*m2 != m2*m1 !!! */
void MMul( Matrix m1, Matrix m2, Matrix res) {
  int i, j;

  // For speed reasons, use the unroll-loops option of your compiler
  for(i = 0; i < 3; i++)
    for(j = 0; j < 3; j++) {
	res[i][j]= m1[i][0]*m2[0][j] + m1[i][1]*m2[1][j] + m1[i][2]*m2[2][j];
	
    }
}

void M4Mul( Matrix4 m1, Matrix4 m2, Matrix4 res) {
  int i, j;

  // For speed reasons, use the unroll-loops option of your compiler
  for(i = 0; i < 4; i++)
    for(j = 0; j < 4; j++) {
	res[i][j]= 
		  m1[i][0]*m2[0][j] 
		+ m1[i][1]*m2[1][j] 
		+ m1[i][2]*m2[2][j] 
		+ m1[i][3]*m2[3][j];
	
    }
}


void MMulV( Matrix m, Vect *v, Vect *r ) {

#ifdef _X86_OPTIMS_
    __asm {
	mov	ebx, DWORD PTR m
	mov	ecx, DWORD PTR v
	mov	eax, DWORD PTR r
    fld   dword ptr [ecx]
    fmul  dword ptr [ebx+24]
    fld   dword ptr [ecx]
    fmul  dword ptr [ebx+12]
    fld   dword ptr [ecx]
    fmul  dword ptr [ebx]
    fld   dword ptr [ecx+4]
    fmul  dword ptr [ebx+4]
    fld   dword ptr [ecx+4]
    fmul  dword ptr [ebx+16]
    fld   dword ptr [ecx+4]
    fmul  dword ptr [ebx+28]
    fxch  ST(2)
    faddp ST(3),ST
    faddp ST(3),ST
    faddp ST(3),ST
    fld   dword ptr [ecx+8]
    fmul  dword ptr [ebx+8]
    fld   dword ptr [ecx+8]
    fmul  dword ptr [ebx+20]
    fld   dword ptr [ecx+8]
    fmul  dword ptr [ebx+32]
    fxch  ST(2)
    faddp ST(3),ST
    faddp ST(3),ST
    faddp ST(3),ST
    fstp  dword ptr [eax]
    fstp  dword ptr [eax+4]
    fstp  dword ptr [eax+8]
    }
#else
    r->x= m[0][0]*v->x + m[0][1]*v->y + m[0][2]*v->z;
    r->y= m[1][0]*v->x + m[1][1]*v->y + m[1][2]*v->z;
    r->z= m[2][0]*v->x + m[2][1]*v->y + m[2][2]*v->z;
#endif

}

int NbM4MulV=0;

#ifdef _X86_OPTIMS_
// Merci intel :-)
void __fastcall M4MulV( Matrix4 m, Vect *v, Vect *r) {
//	c.x = Dot3f(a.data(), data[0]) + data[0][3];
//  c.y = Dot3f(a.data(), data[1]) + data[1][3];
//  c.z = Dot3f(a.data(), data[2]) + data[2][3];
//	c.w = Dot3f(a.data(), data[3]) + data[3][3];
	NbM4MulV++;
	__asm mov	eax, DWORD PTR m
	__asm mov	ebx, DWORD PTR v 
	__asm mov	ecx, DWORD PTR r

    __asm fld   DWORD PTR [ebx]      
    __asm fmul  DWORD PTR [eax+32]  
    __asm fld   DWORD PTR [ebx]      
    __asm fmul  DWORD PTR [eax+16]   
    __asm fld   DWORD PTR [ebx]      
    __asm fmul  DWORD PTR [eax]     
    __asm fld   DWORD PTR [ebx+4]    
    __asm fmul  DWORD PTR [eax+4]  
    __asm fld   DWORD PTR [ebx+4]    
    __asm fmul  DWORD PTR [eax+20]  
    __asm fld   DWORD PTR [ebx+4]    
    __asm fmul  DWORD PTR [eax+36]  
    __asm fxch  ST(2)           
    __asm faddp ST(3),ST       
    __asm faddp ST(3),ST       
    __asm faddp ST(3),ST       
    __asm fld   DWORD PTR [ebx+8]   
    __asm fmul  DWORD PTR [eax+8]  
    __asm fld   DWORD PTR [ebx+8]   
    __asm fmul  DWORD PTR [eax+24]  
    __asm fld   DWORD PTR [ebx+8]   
    __asm fmul  DWORD PTR [eax+40]  
    __asm fxch  ST(2)           
    __asm faddp ST(3),ST       
    __asm faddp ST(3),ST       
    __asm faddp ST(3),ST       
    __asm fadd  DWORD PTR [eax+12]  
    __asm fstp  DWORD PTR [ecx]     
    __asm fadd  DWORD PTR [eax+28]  
    __asm fstp  DWORD PTR [ecx+4]   
    __asm fadd  DWORD PTR [eax+44]  
    __asm fstp  DWORD PTR [ecx+8]  
}

#else
void M4MulV( Matrix4 m, Vect *v, Vect *r ) {
	NbM4MulV++;
    r->x= m[0][0]*v->x + m[0][1]*v->y + m[0][2]*v->z + m[0][3];
    r->y= m[1][0]*v->x + m[1][1]*v->y + m[1][2]*v->z + m[1][3];
    r->z= m[2][0]*v->x + m[2][1]*v->y + m[2][2]*v->z + m[2][3];
}
#endif

void MI_MulV( Matrix m, Vect *v, Vect *r ) {
    r->x= m[0][0]*v->x + m[1][0]*v->y + m[2][0]*v->z;
    r->y= m[0][1]*v->x + m[1][1]*v->y + m[2][1]*v->z;
    r->z= m[0][2]*v->x + m[1][2]*v->y + m[2][2]*v->z;
}

void MAdd( Matrix m1, Matrix m2, Matrix res) {
    int i;

    for (i=2; i>=0; i--) {
	res[i][0] = m1[i][0] + m2[i][0];
	res[i][1] = m1[i][1] + m2[i][1];
	res[i][2] = m1[i][2] + m2[i][2];
    }
}

void MTranspose( Matrix m1 ) {
    Matrix tmp;

    memcpy( tmp, m1, sizeof(Matrix));

    m1[0][1] = tmp[1][0];
    m1[0][2] = tmp[2][0];

    m1[1][0] = tmp[0][1];
    m1[1][2] = tmp[2][1];

    m1[2][0] = tmp[0][2];
    m1[2][1] = tmp[1][2];
}    

void M4Transpose( Matrix4 m, Matrix4 dest ) {

	dest[0][0] = m[0][0];
    dest[0][1] = m[1][0];
    dest[0][2] = m[2][0];
	dest[0][3] = m[3][0];

    dest[1][0] = m[0][1];
    dest[1][1] = m[1][1];
    dest[1][2] = m[2][1];
    dest[1][3] = m[3][1];

    dest[2][0] = m[0][2];
    dest[2][1] = m[1][2];
    dest[2][2] = m[2][2];
    dest[2][3] = m[3][2];

	dest[3][0] = m[0][3];
	dest[3][1] = m[1][3];
	dest[3][2] = m[2][3];
    dest[3][3] = m[3][3];
}    


void QFromRotation( Quatern *q,Vect *v, float fTheta )
{
    q->x = sinf( fTheta/2.0f ) * v->x;
    q->y = sinf( fTheta/2.0f ) * v->y;
    q->z = sinf( fTheta/2.0f ) * v->z;
    q->w = cosf( fTheta/2.0f );
}

//-----------------------------------------------------------------------------
// Name: D3DMath_SlerpQuaternions()
// Desc: Compute a quaternion which is the spherical linear interpolation
//       between two other quaternions by dvFraction.
//-----------------------------------------------------------------------------
void QSlerp(Quatern *q1, Quatern *q2, Quatern *res, float fAlpha)
{
    // Compute dot product (equal to cosine of the angle between quaternions)
    float fCosTheta = q1->x*q2->x + q1->y*q2->y + q1->z*q2->z + q1->w*q2->w;

    // Check angle to see if quaternions are in opposite hemispheres
    if( fCosTheta < 0.0f ) 
    {
        // If so, flip one of the quaterions
        fCosTheta = -fCosTheta;
        q2->x = -q2->x; q2->y = -q2->y; q2->z = -q2->z; q2->w = -q2->w;
    }

    // Set factors to do linear interpolation, as a special case where the
    // quaternions are close together.
    float fBeta = 1.0f - fAlpha;
    
    // If the quaternions aren't close, proceed with spherical interpolation
    if( 1.0f - fCosTheta > 0.001f ) 
    {   
        float fTheta = (float) acos( fCosTheta );
        
        fBeta  = sinf( fTheta*fBeta ) / sinf( fTheta);
        fAlpha = sinf( fTheta*fAlpha ) / sinf( fTheta);
    }

    // Do the interpolation
    res->x = fBeta*q1->x + fAlpha*q2->x;
    res->y = fBeta*q1->y + fAlpha*q2->y;
    res->z = fBeta*q1->z + fAlpha*q2->z;
    res->w = fBeta*q1->w + fAlpha*q2->w;
}



//-----------------------------------------------------------------------------
// Name: D3DMath_MatrixFromQuaternion()
// Desc: Converts a unit quaternion into a rotation matrix.
//-----------------------------------------------------------------------------
void M4FromQ( Matrix4 mat, Quatern q)
{
    float xx = q.x*q.x; float yy = q.y*q.y; float zz = q.z*q.z;
    float xy = q.x*q.y; float xz = q.x*q.z; float yz = q.y*q.z;
    float wx = q.w*q.x; float wy = q.w*q.y; float wz = q.w*q.z;
    
    mat[0][0] = 1 - 2 * ( yy + zz ); 
    mat[0][1] =     2 * ( xy - wz );
    mat[0][2] =     2 * ( xz + wy );

    mat[1][0] =     2 * ( xy + wz );
    mat[1][1] = 1 - 2 * ( xx + zz );
    mat[1][2] =     2 * ( yz - wx );

    mat[2][0] =     2 * ( xz - wy );
    mat[2][1] =     2 * ( yz + wx );
    mat[2][2] = 1 - 2 * ( xx + yy );

    mat[0][3] = mat[1][3] = mat[2][3] = 0.0f;
    mat[3][0] = mat[3][1] = mat[3][2] = 0.0f;
    mat[3][3] = 1.0f;
}

/*
 * Convert a rotation matrix to a quaternion
 */
void QFromM4( Quatern *q, Matrix4 mat) {

	// trace of the matrix
        float T = mat[0][0] + mat[1][1] + mat[2][2] + 1;

	if (T > 0) {

		float S = 0.5 / sqrt(T);

		q->w = 0.25 / S;
		q->x = ( mat[2][1] - mat[1][2] ) * S;
		q->y = ( mat[0][2] - mat[2][0] ) * S;
		q->z = ( mat[1][0] - mat[0][1] ) * S;
		return;
	}


	/*
	 * If the trace of the matrix is less than or equal to zero
	 * then identify which major diagonal element has the greatest
	 * value.
	 */

	if (mat[0][0] > mat[1][1] && mat[0][0] > mat[2][2]) {

		float S  = sqrt( 1.0 + mat[0][0] - mat[1][1] - mat[2][2] ) * 2;

		q->x = 0.5 / S;
		q->y = (mat[0][1] + mat[1][0] ) / S;
		q->z = (mat[0][2] + mat[2][0] ) / S;
		q->w = (mat[1][2] + mat[2][1] ) / S;
		return;
	} 

	if (mat[1][1] > mat[0][0] && mat[1][1] > mat[2][2]) {

		float S  = sqrt( 1.0 + mat[1][1] - mat[0][0] - mat[2][2] ) * 2;

		q->x = (mat[0][1] + mat[1][0] ) / S;
		q->y = 0.5 / S;
		q->z = (mat[1][2] + mat[2][1] ) / S;
		q->w = (mat[0][2] + mat[2][0] ) / S;
		return;
	}

	
        float S  = sqrt( 1.0 + mat[2][2] - mat[0][0] - mat[1][1] ) * 2;

        q->x = (mat[0][2] + mat[2][0] ) / S;
        q->y = (mat[1][2] + mat[2][1] ) / S;
        q->z = 0.5 / S;
        q->w = (mat[0][1] + mat[1][0] ) / S;
}

//-----------------------------------------------------------------------------
// Name: D3DMath_QuaternionMultiply()
// Desc: Mulitples two quaternions together as in {Q} = {A} * {B}.
//-----------------------------------------------------------------------------
void QMul(Quatern *q1, Quatern *q2, Quatern *res)
{
    float x =  q1->x*q2->w + q1->y*q2->z - q1->z*q2->y + q1->w*q2->x;
    float y = -q1->x*q2->z + q1->y*q2->w + q1->z*q2->x + q1->w*q2->y;
    float z =  q1->x*q2->y - q1->y*q2->x + q1->z*q2->w + q1->w*q2->z;
    float w = -q1->x*q2->x - q1->y*q2->y - q1->z*q2->z + q1->w*q2->w;

    res->x = x;
	res->y = y;
	res->z = z;
	res->w = w;
}

void MRotX(Matrix m, float angle) {
	float c = (float) cos(angle);
    float s = (float) sin(angle);

    m[0][0]=1;
    m[0][1]=0;
    m[0][2]=0;
    
    m[1][0]=0;
    m[1][1]=c;
    m[1][2]=-s;

    m[2][0]=0;
    m[2][1]=s;
    m[2][2]=c;
}

void M4RotX(Matrix4 m, float angle) {
	float c = (float) cos(angle);
    float s = (float) sin(angle);

    M4Ident(m);
    
    m[1][1]=c;
    m[1][2]=-s;

    m[2][1]=s;
    m[2][2]=c;
}

void TRotX( Affin *T, float angle) {
    Matrix r, res;
    Vect tvect;

	MRotX(r, angle);

    MMul( r,T->m, res ); /* r <-> m ? */
    memcpy(T->m, res, sizeof(Matrix));
    memcpy(&tvect, &T->v, sizeof(Vect)); 
//    MTranspose(r);
    MMulV( r, &tvect, &T->v);
}

void MRotY(Matrix m, float angle) {
    float c = (float) cos(angle);
    float s = (float) sin(angle);

    m[0][0]=c;
    m[0][1]=0;
    m[0][2]=-s;
    
    m[1][0]=0;
    m[1][1]=1;
    m[1][2]=0;

    m[2][0]=s;
    m[2][1]=0;
    m[2][2]=c;
}

void M4RotY(Matrix4 m, float angle) {
    float c = (float) cos(angle);
    float s = (float) sin(angle);

	M4Ident(m);
    m[0][0]=c;
    m[0][2]=-s;
    
    m[2][0]=s;
    m[2][2]=c;
}


void TRotY( Affin *T, float angle) {
    Affin r,tmp;

	MRotY(r.m, angle);

    r.v.x = r.v.y = r.v.z = 0;
    memcpy(&tmp, T, sizeof(Affin));
    TCompose( &r, &tmp, T);

}

void MRotZ(Matrix m, float angle) {
	float c = (float) cos(angle);
    float s = (float) sin(angle);

    m[0][0]=c;
    m[0][1]=-s;
    m[0][2]=0;
    
    m[1][0]=s;
    m[1][1]=c;
    m[1][2]=0;

    m[2][0]=0;
    m[2][1]=0;
    m[2][2]=1;
}

void M4RotZ(Matrix4 m, float angle) {
	float c = (float) cos(angle);
    float s = (float) sin(angle);

	M4Ident(m);
    m[0][0]=c;
    m[0][1]=-s;
    
    m[1][0]=s;
    m[1][1]=c;
}

void TRotZ( Affin *T, float angle) {
    Matrix r, res;
    Vect tvect;
	
	MRotZ(r, angle);
    MMul( r, T->m, res ); /* r <-> m ? */
    memcpy(T->m, res, sizeof(Matrix));
    memcpy(&tvect, &T->v, sizeof(Vect)); 
    MMulV( r, &tvect, &T->v);
}

/*
void TPrint( Affin *T ) {
    printf("| %5.2f  %5.2f  %5.2f |     | %5.2f |\n",
    	T->m[0][0], T->m[0][1], T->m[0][2], T->v.x);
    printf("| %5.2f  %5.2f  %5.2f |  +  | %5.2f |\n",
    	T->m[1][0], T->m[1][1], T->m[1][2], T->v.y);
    printf("| %5.2f  %5.2f  %5.2f |     | %5.2f |\n",
    	T->m[2][0], T->m[2][1], T->m[2][2], T->v.z);
}
*/

void VPrint( char *str, Vect v ) {
    sprintf(str, "| %3.4f  %3.4f  %3.4f |\n",
		v.x,v.y,v.z);
    	
}    

void M4Print( char *str, Matrix4 m ) {
    sprintf(str, "| %3.4f  %3.4f  %3.4f %3.4f |\n| %3.4f  %3.4f  %3.4f %3.4f |\n| %3.4f  %3.4f  %3.4f %3.4f |\n| %3.4f  %3.4f  %3.4f %3.4f |\n",
    	m[0][0], m[0][1], m[0][2], m[0][3],
    	m[1][0], m[1][1], m[1][2], m[1][3],
    	m[2][0], m[2][1], m[2][2], m[2][3],
    	m[3][0], m[3][1], m[3][2], m[3][3]
		);
}    
/* res(x) = A( B(x) ) */
void TCompose( Affin *A, Affin *B, Affin *res) {
    Vect v;

    MMul( A->m, B->m, res->m );
    MMulV( A->m, &B->v, &v );
    VAdd( &v, &A->v, &res->v );
}

void TApplyV( Affin *T, Vect *v, Vect *res ) {

    MMulV( T->m, v, res);
    VAdd( &T->v, res, res);
}

    
void MGetCam( Vect *r, Matrix mat ) {
    Vect v, cp;
	
	mat[2][0]=r->x;
	mat[2][1]=r->y;
	mat[2][2]=r->z;
	
	v.x = -r->y*r->x;
	v.y = 1-r->y*r->y;
	v.z = -r->y * r->z;
	VNormalize(&v);
	
	mat[1][0] = v.x;
	mat[1][1] = v.y;
	mat[1][2] = v.z;
	VCross(&v, r, &cp);

	mat[0][0]= cp.x;
	mat[0][1]= cp.y;
	mat[0][2]= cp.z;
	
}
