/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
	matvect.h: vector + matrix operations.
	Leto / Calodox, 1998.
*/

#ifndef _MATVECT_H_
#define _MATVECT_H_

#include <math.h>

#define sinf (float) sin
#define cosf (float) cos
#define sqrtf (float) sqrt

/* Macro part */

#define MVScalar(a,b) ((a).x*(b).x + (a).y*(b).y + (a).z*(b).z)
#define MVMul(fact, v) { v.x *= fact; v.y *= fact; v.z *= fact; }
#define MVAdd(v1, v2, res) { res.x = v1.x + v2.x; res.y = v1.y + v2.y; res.z = v1.z + v2.z; }

typedef struct {
	float x,y;
} Vect2D;

typedef struct {
    float x,y,z;
} Vect;

typedef struct {
	float x,y,z,w;
} Quatern;

/* Matrix[y][x] */
typedef float Matrix[3][3];
typedef float Matrix4[4][4];

typedef struct {
    Vect v;
    Matrix m;
} Affin; /* is it the right word in english ? */

/* prefix system:
	V.. -> vectorial operation.
	M.. -> matrix operation.
	M4..-> homogeneous matrix operation 
	T.. -> affin transformation operation.
*/

extern int NbM4MulV;

/* vector operations: */

void VAdd( Vect *v1, Vect *v2, Vect *res );
void VSub( Vect *v1, Vect *v2, Vect *res );

void VMul( float fact, Vect *v );
float VScalar( Vect *v1, Vect *v2 );
void VCross( Vect *v1, Vect *v2, Vect *res );
void VNormalize( Vect *v1 );
void VInvert( Vect *v );
float VLength( Vect *v );

/* matrix operations: */

void MIdent( Matrix m );
void M4Ident( Matrix4 m);
void M4Print( char *str, Matrix4 m );
void VPrint( char *str, Vect v );

/* m1*m2 != m2*m1 !!! */
void MMul( Matrix m1, Matrix m2, Matrix res);
void M4Mul( Matrix4 m1, Matrix4 m2, Matrix4 res);
void MMulV( Matrix m, Vect *v, Vect *r );

# ifdef _X86_OPTIMS_
void __fastcall M4MulV( Matrix4 m, Vect *v, Vect *r );
#else
void M4MulV( Matrix4 m, Vect *v, Vect *r );
#endif 

void MI_MulV( Matrix m, Vect *v, Vect *r );
void MAdd( Matrix m1, Matrix m2, Matrix res);
void MTranspose( Matrix m1 );
void M4Transpose( Matrix4 m, Matrix4 dest );

// Rotations and quaternions
void M4FromQ( Matrix4 mat, Quatern q);
void QFromM4( Quatern *q, Matrix4 mat);
void QFromRotation( Quatern *q,Vect *v, float fTheta );
void QSlerp(Quatern *q1, Quatern *q2, Quatern *res, float fAlpha);
void QMul(Quatern *q1, Quatern *q2, Quatern *res);


// remplit m avec une matrice de rotation
void MRotX(Matrix m, float angle);
void MRotY(Matrix m, float angle);
void MRotZ(Matrix m, float angle);

void M4RotX(Matrix4 m, float angle);
void M4RotY(Matrix4 m, float angle);
void M4RotZ(Matrix4 m, float angle);

void TRotX( Affin *T, float angle);
void TRotY( Affin *T, float angle);
void TRotZ( Affin *T, float angle);

void TPrint( Affin *T );
/* res = A(B) */
void TCompose( Affin *A, Affin *B, Affin *res);
void TApplyV( Affin *T, Vect *v, Vect *res );

void MGetCam( Vect *r, Matrix mat );


#endif

